//
//  The following unit is a DELPHI interfact into the Intel Singal Processing
//  library Version 4.0.  The library can be obtained from
//
//    http://developer.intel.com/design/perftool/PERFLIBST/spl/index.htm
//
//  The signal processing library is Copyright (c) 1995 Intel Corporation. All Rights Reserved.
//
//  This unit is just the header files of the library translated from C to Delphi
//  I make no claims to the correctness of the library, this unit and take
//  no responsiblity for its use
//
//                                       -John_Mertus@Brown.EDU
//
            unit IntelNSP4;

     interface

Uses Windows;


//=== Definitions =========================================================

Const
    NSP_EPS  = 1.0E-12;
    NSP_PI   = 3.14159265358979324;
    NSP_2PI  = 6.28318530717958648;
    NSP_PI_2 = 1.57079632679489662;
    NSP_PI_4 = 0.785398163397448310;

//--- Common Types --------------------------------------------------------

Type
  pSCplx = ^SCplx;
  SCplx  = Record
    re : Single;
    im : Single;
  End;

  pDCplx = ^DCplx;
  DCplx  = Record
    re : Double;
    im : Double;
  End;

  pWCplx = ^WCplx;
  WCplx  = Record
    re : SmallInt;
    im : SmallInt;
  End;

  pICplx = ^ICplx;
  ICplx  = Record
    re : LongInt;
    im : LongInt;
  End;

type NSPAutoCorrType = (NSP_Normal, NSP_Biased, NSP_UnBias);


//=== NSPAPI Definition ===================================================



   Function nspMalloc(len : Integer) : Pointer; StdCall; external 'nsp.dll';
   Function nspsMalloc(len : Integer) : pInteger; StdCall; external 'nsp.dll';
   Function nspdMalloc(len : Integer) : pdouble; StdCall; external 'nsp.dll';
   Function nspcMalloc(len : Integer) : pSCplx; StdCall; external 'nsp.dll';
   Function nspzMalloc(len : Integer) : pDCplx; StdCall; external 'nsp.dll';

   Function nspwMalloc(len : Integer) : pSmallInt; StdCall; external 'nsp.dll';
   Function nspvMalloc(len : Integer) : pWCplx; StdCall; external 'nsp.dll';
   Function nspiMalloc(len : Integer) : pLongInt; StdCall; external 'nsp.dll';
   Function nspjMalloc(len : Integer) : pICplx; StdCall; external 'nsp.dll';

   Procedure nspFree(ptr : Pointer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//   Miscellaneous Scalar Functions and Vector Functions
//        Complex Add, Sub, Mpy, Div, Conj
//
//   These functions perform addition, subtraction, multiplication,
//   division, and conjugation on complex numbers a and b.

   Function nspcAdd(a : SCplx; b : SCplx) : SCplx; StdCall; external 'nsp.dll';
   Function nspcSub(a : SCplx; b : SCplx) : SCplx; StdCall; external 'nsp.dll';
   Function nspcMpy(a : SCplx; b : SCplx) : SCplx; StdCall; external 'nsp.dll';
   Function nspcDiv(a : SCplx; b : SCplx) : SCplx; StdCall; external 'nsp.dll';
   Function nspcConj(a : SCplx) : SCplx; StdCall; external 'nsp.dll';

//---- Additional Functions -----------------------------------------------

   Function nspcSet(re, im : Single) : SCplx; StdCall; external 'nsp.dll'

//-------------------------------------------------------------------------
//        Vector Initialization
//   These functions initialize vectors of length n.

   Procedure nspsbZero(dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbZero(dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsbSet(val : Single; dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSet(re, im : Single;  dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsbCopy(src : pSingle; dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbCopy(src : pSCplx;  dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//        Vector Addition and multiplication                               
//   These functions perform element-wise arithmetic on vectors of length n

// dst[i]=dst[i]+val;                                                      

   Procedure nspsbAdd1(val : Single; dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbAdd1(val : SCplx; dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]+src[i];

   Procedure nspsbAdd2(src : pSingle; dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbAdd2(src : pSCplx;  dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=srcA[i]+srcB[i];                                                 

   Procedure nspsbAdd3(srcA, srcB : pSingle; dst : pSingle;n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbAdd3(srcA, srcB : pSCplx;  dst : pSCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]-val;                                                      

   Procedure nspsbSub1(val : Single; dst : pSingle;n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSub1(val : SCplx; dst : pSCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]-val[i];

   Procedure nspsbSub2(val : psingle; dst : pSingle;n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSub2(val : pSCplx; dst : pSCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=src[i]-val[i];

   Procedure nspsbSub3(src : pSingle; val : psingle;  dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSub3(src : pSCplx;  val : pSCplx;  dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]*val;

   Procedure nspsbMpy1(val : Single; dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbMpy1(val : SCplx; dst : pSCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]*src[i];

   Procedure nspsbMpy2(src : pSingle; dst : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbMpy2(src : pSCplx; dst : pSCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=srcA[i]*srcB[i];

   Procedure nspsbMpy3(srcA, srcB : pSingle; dst : pSingle;n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbMpy3(srcA, srcB : pSCplx;  dst : pSCplx;n : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//        Complex conjugates of scalars and vectors

   Procedure  nspcbConj1(vec : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcbConj2(src : pSCplx;  dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcbConjFlip2(src : pSCplx;  dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcbConjExtend1(vec : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcbConjExtend2(src : pSCplx;  dst : pSCplx; n : Integer);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
//   Miscellaneous Scalar Functions and Vector Functions
//        Complex Add, Sub, Mpy, Div, Conj
//   These functions perform addition, subtraction, multiplication,
//   division, and conjugation on complex numbers a and b.
//

   Function nspzAdd(a, b : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspzSub(a, b : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspzMpy(a, b : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspzDiv(a, b : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspzConj(a : DCplx): DCplx; StdCall; external 'nsp.dll';

//---- Additional Functions -----------------------------------------------

   Function nspzSet(re, im : Double): DCplx; StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//        Vector Initialization
//   These functions initialize vectors of length n.

   Procedure  nspdbZero(dst : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbZero(dst : pDCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspdbSet(val : double; dst : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbSet(re, im : double; dst : pDCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspdbCopy(src : pdouble; dst : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbCopy(src : pDCplx; dst : pDCplx; n : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//        Vector Addition and multiplication
//   These functions perform element-wise arithmetic on vectors of length n

// dst[i]=dst[i]+val;

   Procedure  nspdbAdd1(val : double; dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbAdd1(val : DCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';
// dst[i]=dst[i]+src[i];

   Procedure  nspdbAdd2(src : pdouble; dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbAdd2(src : pDCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=srcA[i]+srcB[i];

   Procedure  nspdbAdd3(srcA, srcB : pdouble;dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbAdd3(srcA, srcB : pDCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]-val;

   Procedure  nspdbSub1(val : double; dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbSub1(val : DCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]-val[i];

   Procedure  nspdbSub2(val : pdouble; dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbSub2(val : pDCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=src[i]-val[i];             

   Procedure  nspdbSub3(src : pdouble; val : pdouble;  dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbSub3(src : pDCplx; val : pDCplx;  dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]*val;                

   Procedure  nspdbMpy1(val : double; dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbMpy1(val : DCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';

// dst[i]=dst[i]*src[i];             

   Procedure  nspdbMpy2(src : pdouble;dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbMpy2(src : pDCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';
    
// dst[i]=srcA[i]*srcB[i];           

   Procedure  nspdbMpy3(srcA, srcB : pdouble;dst : pDouble;n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbMpy3(srcA, srcB : pDCplx; dst : pDCplx;n : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//
//        Complex conjugates of scalars and vectors     
//

   Procedure  nspzbConj1(vec : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbConj2(src : pDCplx; dst : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbConjFlip2(src : pDCplx; dst : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbConjExtend1(vec : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbConjExtend2(src : pDCplx; dst : pDCplx; n : Integer);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
//   Miscellaneous Scalar Functions and Vector Functions
//        Complex Add, Sub, Mpy, Div, Conj
//   These functions perform addition, subtraction, multiplication,
//   division, and conjugation on complex numbers a and b.
//                

   Function  nspvAdd(a, b : WCplx;
           scaleMode : Integer; var scaleFactor : Integer): WCplx; StdCall; external 'nsp.dll';
   Function nspvSub(a, b : WCplx;
           scaleMode : Integer; var scaleFactor : Integer): WCplx; StdCall; external 'nsp.dll';
   Function nspvMpy(a, b : WCplx;
           scaleMode : Integer; var scaleFactor : Integer): WCplx; StdCall; external 'nsp.dll';
   Function nspvDiv(a : WCplx; b : WCplx): WCplx; StdCall; external 'nsp.dll';
   Function nspvConj(a : WCplx): WCplx; StdCall; external 'nsp.dll';

//---- Additional Functions -----------------------------------------------

   Function nspvSet(re, im : SmallInt): WCplx; StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//        Vector Initialization      
//   These functions initialize vectors of length n.    

   Procedure  nspwbZero(dst : pSmallint; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbZero(dst : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbSet(val : smallint; dst : pSmallint; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbSet(re, im : SmallInt; dst : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbCopy(src : pSmallInt; dst : pSmallint; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbCopy(src : pWCplx; dst : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//        Vector Addition and multiplication            
//   These functions perform element-wise arithmetic on vectors of length n

   Procedure  nspwbAdd1(val : SmallInt; dst : pSmallint; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbAdd1(val : WCplx; dst : pWCplx; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbAdd2(src : pSmallInt;dst : pSmallint; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbAdd2(src : pWCplx; dst : pWCplx; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbAdd3(srcA, srcB : pSmallInt; dst : pSmallint; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbAdd3(srcA, srcB : pWCplx; dst : pWCplx; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbSub1(val : SmallInt; dst : pSmallint; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbSub1(val : WCplx; dst : pWCplx; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbSub2(val : pSmallInt; dst : pSmallint; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbSub2(val : pWCplx;  dst : pWCplx; n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbSub3(src : pSmallInt; val : pSmallInt;  dst : pSmallint;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbSub3(src : pWCplx; val : pWCplx;   dst : pWCplx;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbMpy1(val : SmallInt; dst : pSmallint;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbMpy1(val : WCplx; dst : pWCplx;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbMpy2(src : pSmallInt;dst : pSmallint;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbMpy2(src : pWCplx;dst : pWCplx;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbMpy3(srcA, srcB : pSmallInt; dst : pSmallint;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbMpy3(srcA, srcB : pWCplx; dst : pWCplx;n : Integer;
           scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//        Complex conjugates of vectors

   Procedure  nspvbConj1(vec : pWCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbConj2(src : pWCplx; dst : pWCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbConjFlip2(src : pWCplx; dst : pWCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbConjExtend1(vec : pWCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbConjExtend2(src : pWCplx; dst : pWCplx; n : Integer);  StdCall; external 'nsp.dll';



{
 * Conv2D
 *
 * Perform finite, linear convolution of two two-dimensional signals.
}

   Procedure nspsConv2D(x: pSingle; xCols, xRows : Integer;
     var h : Single; hCols, hRows : Integer; var y : Single);  StdCall; external 'nsp.dll';
   Procedure nspdConv2D(x : pDouble;  xCols, xRows : Integer;
     var h : Double;  hCols, hRows : Integer; var y : Double);  StdCall; external 'nsp.dll';

   Procedure nspwConv2D(var x : Smallint; xCols, xRows : Integer;
     var h : Smallint; hCols, hRows : Integer; var y : Smallint;
           scaleMode : Integer; var scaleFactorPtr : Integer);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
//         Conv   
//                
// Performs finite, linear convolution of two sequences.

   Procedure nspsConv(x: pSingle; xLen : Integer;
    var h : Single; hLen : Integer; var y : Single);  StdCall; external 'nsp.dll';

   Procedure nspcConv(x : pSCplx; xLen : Integer;
    var h : SCplx; hLen : Integer; var y : SCplx);  StdCall; external 'nsp.dll';

   Procedure nspdConv(x : pDouble;  xLen : Integer;
    var h : Double;  hLen : Integer; var y : Double);  StdCall; external 'nsp.dll';

   Procedure nspzConv(x : pDCplx; xLen : Integer;
    var h : DCplx; hLen : Integer; var y : DCplx);  StdCall; external 'nsp.dll';

   Procedure nspscConv(x: pSingle; xLen : Integer;
    var h : SCplx; hLen : Integer; var y : SCplx);  StdCall; external 'nsp.dll';

   Procedure nspcsConv(x : pSCplx; xLen : Integer;
    var h : Single; hLen : Integer; var y : SCplx);  StdCall; external 'nsp.dll';
    
   Procedure nspdzConv(x : pDouble;  xLen : Integer;
    var h : DCplx; hLen : Integer; var y : DCplx);  StdCall; external 'nsp.dll';

   Procedure nspzdConv(x : pDCplx; xLen : Integer;
    var h : Double;  hLen : Integer; var y : DCplx);  StdCall; external 'nsp.dll';

   Procedure nspwConv(var x : Smallint; xLen : Integer;
    var h : Smallint; hLen : Integer; var y : Smallint;
          scaleMode : Integer;  var scaleFactor : Integer);  StdCall; external 'nsp.dll';



// ------------------------------------------------------------------------
//        AutoCorrelation
//                
{
  Normal:
  ~~~~~~~
  dst[n] = S ~src[k] * src[k+n],  0 <= n < nLags
               0 <= k < len

  Biased:
  ~~~~~~~
  dst[n] = K* S ~src[k] * src[k+n],  0 <= n < nLags
                  0 <= k < len
                  K = 1/len
  UnBiased:
  ~~~~~~~~~
  dst[n] = K* S ~src[k] * src[k+n],  0 <= n < nLags
                  0 <= k < len
                  K = 1/(len-n)

       | src[k], 0<=k<len
                src(k) = <
       | 0, otherwise

}
   Procedure  nspsAutoCorr(src : pSingle; len : integer; dst : pSingle; nlags : Integer;
              NSPcorrType : NSPAutoCorrType);  StdCall; external 'nsp.dll';
   Procedure  nspcAutoCorr(src : pSCplx;  len : integer; dst : pSCplx; nlags : Integer;
              NSPcorrType : NSPAutoCorrType);  StdCall; external 'nsp.dll';
   Procedure  nspdAutoCorr(src : pdouble; len : integer; dst : pDouble; nlags : Integer;
              NSPcorrType : NSPAutoCorrType);  StdCall; external 'nsp.dll';
   Procedure  nspzAutoCorr(src : pDCplx; len : integer; dst : pDCplx; nlags : Integer;
              NSPcorrType : NSPAutoCorrType);  StdCall; external 'nsp.dll';
   Procedure  nspwAutoCorr(src : pSmallInt; len : integer; dst : pSmallint; nlags : Integer;
              corrType : NSPAutoCorrType;
              scaleMode : Integer; var scalePtr : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvAutoCorr(src : pWCplx; len : integer; dst : pWCplx; nlags : Integer;
              corrType : NSPAutoCorrType;
              scaleMode : Integer; var scalePtr : Integer);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        CrossCorralation
{
  dst[n] = S ~srcA[k] *srcB[k+n+loLag],  0 <= n <= hiLag-loLag
    0 <= k < lenA

           | srcB[k], 0<=k<lenB
 srcB(k) = <
           | 0, otherwise
  Number of result elements is hiLag-loLag+1.
  Estimate at lag of loLag in dst[0].
  Estimate at lag of hiLag in dst[hiLag-loLag].
}

   Procedure  nspsCrossCorr(srcA : pSingle; lenA : Integer;
          srcB : pSingle;  lenB : Integer; dst : pSingle;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcCrossCorr(srcA : pSCplx;  lenA : Integer;
          srcB : pSCplx;  lenB : Integer; dst : pSCplx;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspscCrossCorr(srcA : pSingle; lenA : Integer;
          srcB : pSCplx;  lenB : Integer; dst : pSCplx;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcsCrossCorr(srcA : pSCplx;  lenA : Integer;
          srcB : pSingle;  lenB : Integer; dst : pSCplx;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';

   Procedure   nspdCrossCorr(srcA : pDouble;  lenA : Integer;
          srcB : pDouble;  lenB : Integer; dst : pDouble;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspzCrossCorr(srcA : pDCplx;  lenA : Integer;
          srcB : pDCplx;  lenB : Integer; dst : pDCplx;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspdzCrossCorr(srcA : pDouble;  lenA : Integer;
          srcB : pDCplx;  lenB : Integer; dst : pDCplx;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzdCrossCorr(srcA : pDCplx;  lenA : Integer;
          srcB : pDouble;  lenB : Integer; dst : pDCplx;
                loLag : Integer; hiLag : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspwCrossCorr(srcA : pSmallInt;  lenA : Integer;
          srcB : pSmallInt;  lenB : Integer; dst : pSmallint;
                loLag : Integer; hiLag : Integer;
                scaleMode : Integer; var scalePtr : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvCrossCorr(srcA : pWCplx;  lenA : Integer;
          srcB : pWCplx;  lenB : Integer; dst : pWCplx;
                loLag : Integer; hiLog : Integer;
                scaleMode : Integer; var scalePtr : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspwvCrossCorr(srcA : pSmallInt;  lenA : Integer;
          srcB : pWCplx;  lenB : Integer; dst : pWCplx;
                loLag : Integer; hiLog : Integer;
                scaleMode : Integer; var scalePtr : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvwCrossCorr(srcA : pWCplx;  lenA : Integer;
          srcB : pSmallInt;  lenB : Integer; dst : pWCplx;
                loLag : Integer; hiLog : Integer;
                scaleMode : Integer; var scalePtr : Integer);  StdCall; external 'nsp.dll';


//-------Flags to determine the transformation-----------------------------

Const

NSP_Noflags  =  0;         // Specifies an absence of all flags

NSP_Round   =  $80;      // Specifies that floating-point values
             // should be rounded to the nearest integer
NSP_TruncZero = $100;      // Specifies that floating-point values
             // should be truncated toward zero
NSP_TruncNeg  = $200;     // Specifies that floating-point values
             // should be truncated toward negative
             // infinity
NSP_TruncPos  = $400;     // Specifies that floating-point values
             // should be truncated toward positive
             // infinity
NSP_Unsigned  = $800;     // Specifies that integer values are
             // unsigned
NSP_Clip  =     $1000;    // Specifies that floating-point values
             // outside the allowable integer range
             // are saturated to maximum(minimum)
             // integer value
NSP_OvfErr  =  $2000;    // Specifies that an overflow error should
             // be signaled with a call to nspError()

//-------------------------------------------------------------------------
// Maximum possible phase value for short function, corresponds to NSP_PI  

NSP_CVRT_MAXPHASE = 16383;

//-------------------------------------------------------------------------
//     Short functions
//-------------------------------------------------------------------------
// FUNCTION:
//   nspvbReal
// DESCRIPTION:
//   Return the real part of the coplex vector
// PARAMETERS:
//   src       - an input complex vector;
//   dst       - an output vector to store the real part;
//   len       - a length of the arrays;
// ERRORS:
//    1) Some of pointers to input or output data are NULL
//    2) The length of the arrays is less or equal zero
//   These errors are registered only if NSP_DEBUG is defined

   Procedure  nspvbReal(src : pWCplx; dst : pSmallint; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:
//   nspvbImag
// DESCRIPTION:
//   Return the imaginary part of the coplex vector     
// PARAMETERS:
//   src       - an input complex vector;               
//   dst       - an output vector to store the imaginary part;
//   len       - a length of the arrays.                
// ERRORS:        
//    1) Some of pointers to input or output data are NULL                 
//    2) The length of the arrays is less or equal zero 
//  These errors are registered only if NSP_DEBUG is defined

   Procedure  nspvbImag(src : pWCplx; dst : pSmallint; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nspvbMag
// DESCRIPTION:
//   Compute the magnitude of complex vector elements.
// PARAMETERS:
//   src       - an input complex vector
//   mag       - an output vector to store the magnitude components;
//   len       - a length of the arrays.
// ALGORITHM:
//   Computation is perfomed using standart C mathematical functions.
// ERRORS:
//    1) Some of pointers to input or output data are NULL
//    2) The length of the arrays is less or equal zero
//  These errors are registered only if NSP_DEBUG is defined

   Procedure  nspvbMag(src : pWCplx; var mag : SmallInt; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:
//   nspvbPhase
// DESCRIPTION:
//   Compute the phase (in radians) of complex vector elements.
// PARAMETERS:
//   src       - an input complex vector
//   phase     - an output vector to store the phase components;
//   len       - a length of the arrays.
// ALGORITHM:
//   Computation is perfomed using standart C mathematical functions.      
//   Phase of (0.0, 0.0) is assumed to be 0.0.
//   Phase values are in the range (-Pi, Pi].           
// ERRORS:        
//    1) Some of pointers to input or output data are NULL                 
//    2) The length of the arrays is less or equal zero 
//  These errors are registered only if NSP_DEBUG is defined               

   Procedure  nspvbPhase(src : pWCplx; var phase : SmallInt; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nspwbrMag    
// DESCRIPTION:   
//   Compute the magnitude of complex data formed as two real vectors.     
// PARAMETERS:    
//   srcReal   - an input vector containing a real part of complex data    
//   srcImag   - an input vector containing an imag. part of complex data  
//   mag       - an output vector to store the magnitude components;       
//   len       - a length of the arrays.                
// ALGORITHM:     
//   Computation is perfomed using standart C mathematical functions.
// ERRORS:        
//    1) Some of pointers to input or output data are NULL                 
//    2) The length of the arrays is less or equal zero 
//  These errors are registered only if NSP_DEBUG is defined               

   Procedure  nspwbrMag(srcReal : pSmallInt; srcImag : pSmallInt;
            mag : pSmallInt; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nspwbrPhase  
// DESCRIPTION:   
//   Compute the phase of complex data formed as two real vectors.         
// PARAMETERS:    
//   srcReal   - an input vector containing a real part of complex data    
//   srcImag   - an input vector containing an imag. part of complex data  
//   phase     - an output vector to store the magnitude components;       
//   len       - a length of the arrays.                
// ALGORITHM:     
//   Computation is perfomed using standart C mathematical functions.
//   Phase of (0.0, 0.0) is assumed to be 0.0.          
//   Phase values are in the range (-Pi, Pi].           
// ERRORS:        
//    1) Some of pointers to input or output data are NULL                 
//    2) The length of the arrays is less or equal zero 
//  These errors are registered only if NSP_DEBUG is defined               

   Procedure  nspwbrPhase(srcReal : pSmallInt; srcImag : pSmallInt;
              phase : pSmallInt; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nspvbCplxTo2Real                
// DESCRIPTION:   
//   Return the real and imaginary parts of the coplex vector              
// PARAMETERS:    
//   src       - an input complex vector                
//   dstReal   - an output vector to store the real part
//   dstImag   - an output vector to store the imaginary part;             
//   len       - a length of the arrays.
// ERRORS:        
//    1) Some of pointers to input or output data are NULL                 
//    2) The length of the arrays is less or equal zero 
//  These errors are registered only if NSP_DEBUG is defined               

   Procedure  nspvbCplxTo2Real(src : pWCplx;     dstReal : pSmallInt;
          dstImag : pSmallInt; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nspvb2RealToCplx                
// DESCRIPTION:   
//   Construct complex vector from real and imaginary components           
// PARAMETERS:    
//   srcReal   - an input real component. May be NULL - a real part of the 
//               output will be zero;
//   srcImag   - an input imaginary component. May be NULL - an imaginary  
//               part of the output will be zero;       
//   dst       - an output complex vector;              
//   len       - a length of the arrays.
// ERRORS:        
//    1) The pointer to output data is NULL             
//    2) The length of the arrays is less or equal zero 
//  These errors are registered only if NSP_DEBUG is defined               

   Procedure  nspvb2RealToCplx(srcReal : pSmallInt; srcImag : pSmallInt;
dst : pWCplx;           len : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//             bFloatToInt, bIntTofloat       
//                
//Vector floating-point to integer and integer to floating-point conversion
//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,d>bFloatToInt             
// DESCRIPTION:   
//   Converts the floating-point data in the input array and stores        
//   the results in the output array.
// PARAMETERS:
//   src       - an input array to be converted;        
//   dst       - an output array to store the result;   
//   len       - a length of the arrays;                
//   wordSize  - specifies the size of an integer in bits;     
//  and must be 8, 16 or 32;
//   flags     - specifies the sort of the conversion and consists of the
//               bitwise-OR of flags, defined in the beginnig of this file
//

   Procedure nspsbFloatToInt(src : pSingle; dst : Pointer; len : integer;
                wordSize : Integer;       flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbFloatToInt(src : pdouble; dst : Pointer; len : integer;
                wordSize : Integer;       flags : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:
//   nsp<s,d>bIntTofloat
// DESCRIPTION:
//   Converts the integer data in the input array and stores the results
//   in the output array
// PARAMETERS:
//   src       - an input array to be converted;
//   dst       - an output array to store the result;
//   len       - a length of the arrays;
//   wordSize  - specifies the size of an integer in bits;
//  and must be 8, 16 or 32;
//   flags     - specifies that integer values are unsigned or signed.
//

   Procedure nspsbIntToFloat(src : pointer; dst : pSingle; len : integer;
                wordSize : Integer;         flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbIntToFloat(src : pointer; dst : pDouble; len : integer;
                wordSize : Integer;         flags : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//             bFloatToFix, bFixTofloat
//
// Vector floating-point to fixed-point and
//            fixed-point to floating-point conversion
//-------------------------------------------------------------------------
// FUNCTION:
//   nsp<s,d>bFloatToFix
// DESCRIPTION:
//   Converts the floating-point data in the input array and stores
//   the results in the output array.
// PARAMETERS:
//   src       - an input array to be converted;
//   dst       - an output array to store the result;
//   len       - a length of the arrays;
//   wordSize  - specifies the size of an fix-point in bits;
//  and must be 8, 16 or 32;
//   fractBits - specifies the number of fractional bits
//   flags     - specifies the sort of the conversion and consists of the
//               bitwise-OR of flags, defined in the beginnig of this file
//

   Procedure nspsbFloatToFix(src : pSingle;     dst : Pointer;      len : integer;
                wordSize : Integer; fractBits : Integer; flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbFloatToFix(src : pdouble;     dst : Pointer;      len : integer;
                wordSize : Integer; fractBits : Integer; flags : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:
//   nsp<s,d>bFixTofloat
// DESCRIPTION:
//   Converts the fixed-point data in the input array and stores the
//       results in the output array
// PARAMETERS:
//   src       - an input array to be converted;
//   dst       - an output array to store the result;   
//   len       - a length of the arrays;                
//   wordSize  - specifies the size of an integer in bits;     
//  and must be 8, 16 or 32;         
//   fractBits - specifies the number of fractional bits
//   flags     - specifies that fixed-point values are unsigned or signed  
//                

   Procedure nspsbFixToFloat(var src;    dst : pSingle;      len : integer;
                wordSize : Integer;  fractBits : Integer; flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbFixToFloat(var src;     dst : pDouble;      len : integer;
                wordSize : Integer;  fractBits : Integer; flags : Integer);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
// bFloatToS31Fix, bFloatToS15Fix, bFloatToS7Fix, bFloatToS1516Fix,        
// bS31FixToFloat, bS15FixToFloat, bS7FixToFloat, bS1516FixTofloat  
// Vector floating-point to fixed-point of format S.31, S.15, S.7, S15.16  
//                and fixed-point of format S.31, S.15, S.7, S15.16        
//                to floating-point conversion          
//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,d>bFloatToS31Fix          
//   nsp<s,d>bFloatToS15Fix          
//   nsp<s,d>bFloatToS7Fix           
//   nsp<s,d>bFloatToS1516Fix        
// DESCRIPTION:   
//   Converts the floating-point data in the input array and stores
//   the results in the output array.
//   nsp?bFloatToS31Fix() to format of S.31 (a sign bit,31 fractional bits)
//   nsp?bFloatToS15Fix() to format of S.15 (a sign bit,15 fractional bits)
//   nsp?bFloatToS7Fix()  to format of S.7  (a sign bit,7  fractional bits)
//   nsp?bFloatToS1516Fix() to format of S15.16         
// (a sign bit, 15 integer bits, 16 fractional bits)    
// PARAMETERS:    
//   src       - an input array to be converted;        
//   dst       - an output array to store the result;   
//   len       - a length of the arrays;                
//   flags     - specifies the sort of the conversion and consists of the  
//               bitwise-OR of flags, defined in the beginnig of this file 

   Procedure nspsbFloatToS31Fix(src : pSingle; dst : pInteger; len : integer;flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbFloatToS15Fix(src : pSingle; dst : pSmallint; len : integer;flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbFloatToS7Fix(src : pSingle; dst : pChar; len : integer;flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbFloatToS1516Fix(src : pSingle; dst : pInteger; len : integer;flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdbFloatToS31Fix(src : pdouble;dst : pInteger;len : integer;flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbFloatToS15Fix(src : pdouble;dst : pSmallint;len : integer;flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbFloatToS7Fix(src : pdouble;dst : pChar;len : integer;flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbFloatToS1516Fix(src : pdouble;dst : pInteger; len : integer;flags : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<s,d>bS31FixTofloat 
//   nsp<s,d>bS15FixTofloat 
//   nsp<s,d>bS7FixTofloat 
//   nsp<s,d>bS1516FixTofloat 
// DESCRIPTION:   
//   Converts the fixed-point data in the input array and stores the       
//       results in the output array 
//  nsp?bS31FixToFloat() from format of S.31(a sign bit,31 fractional bits)
//  nsp?bS15FixToFloat() from format of S.15(a sign bit,15 fractional bits)
//  nsp?bS7FixToFloat()  from format of S.7 (a sign bit,7  fractional bits)
//  nsp?bS1516FixToFloat() from format of S15.16        
// (a sign bit, 15 integer bits, 16 fractional bits)    
// PARAMETERS:    
//   src       - an input array to be converted;        
//   dst       - an output array to store the result;   
//   len       - a length of the arrays;
//                

   Procedure nspsbS31FixToFloat(src : pInteger; dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbS15FixToFloat(src : pSmallInt;dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbS7FixToFloat(src : pChar; dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbS1516FixToFloat(src : pInteger; dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbS31FixToFloat(src : pInteger;  dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbS15FixToFloat(src : pSmallInt; dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbS7FixToFloat(src : pChar;  dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbS1516FixToFloat(src : pInteger;  dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//  bReal, bImag, bCplxTo2Real, b2RealToCplx            
//                
// Return the real and imaginary parts of complex vectors                  
// or construct complex vectors from real and imaginary components         
//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<c,z>bReal
// DESCRIPTION:   
//   Return the real part of the coplex vector          
// PARAMETERS:    
//   src       - an input complex vector                
//   dst       - an output vector to store the real part;                  
//   len       - a length of the arrays;
//                
   Procedure nspcbReal(src : pSCplx; dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbReal(src : pDCplx; dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<c,z>bImag
// DESCRIPTION:   
//   Return the imaginary part of the coplex vector     
// PARAMETERS:    
//   src       - an input complex vector                
//   dst       - an output vector to store the imaginary part;             
//   len       - a length of the arrays;                
//                
   Procedure nspcbImag(src : pSCplx; dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbImag(src : pDCplx;  dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<c,z>bCplxTo2Real            
// DESCRIPTION:   
//   Return the real and imaginary parts of the coplex vector              
// PARAMETERS:    
//   src       - an input complex vector                
//   dstReal   - an output vector to store the real part
//   dstImag   - an output vector to store the imaginary part;             
//   len       - a length of the arrays;                
//                
   Procedure nspcbCplxTo2Real(src : pSCplx; dstReal : pSingle;
                 dstImag : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbCplxTo2Real(src : pDCplx; dstReal : pDouble;
                 dstImag : pDouble; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<c,z>b2RealToCplx            
// DESCRIPTION:   
//   Construct complex vector from real and imaginary components           
// PARAMETERS:    
//   srcReal   - an input real component. May be NULL - a real part of the 
//               output will be zero;
//   srcImag   - an input imaginary component. May be NULL - an imaginary  
//               part of the output will be zero;       
//   dst       - an output complex vector;              
//   len       - a length of the arrays;                
//                
   Procedure nspcb2RealToCplx(srcReal : pSingle; srcImag : pSingle;
                 dst : pSCplx;           len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzb2RealToCplx(srcReal : pdouble; srcImag : pdouble;
                 dst : pDCplx;           len : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//  bCartToPolar, brCartToPolar, bPolarToCart, brPolarToCart               
//                
// Cartesian to polar and polar to cartesian coordinate conversions.       
//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<c,z>bCartToPolar            
// DESCRIPTION:   
//   Convert cartesian coordinate to polar. Input data are formed as       
//            a complex vector.      
// PARAMETERS:
//   src       - an input complex vector;               
//   mag       - an output vector to store the magnitude components;
//   phase     - an output vector to store the phase components (in rad);  StdCall; external 'nsp.dll'; 
//   len       - a length of the arrays;                
//                
   Procedure nspcbCartToPolar(src : pSCplx;  mag : pSingle; phase : pSingle;
                 len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbCartToPolar(src : pDCplx;  mag : pDouble; phase : pDouble;
                 len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<s,d>brCartToPolar           
// DESCRIPTION:   
//   Convert cartesian coordinate to polar. Input data are formed as       
//            two different real vectors.               
// PARAMETERS:    
//   srcReal   - an input vector containing the coordinates X;             
//   srcImag   - an input vector containing the coordinates Y;             
//   mag       - an output vector to store the magnitude components;       
//   phase     - an output vector to store the phase components (in rad);  StdCall; external 'nsp.dll'; 
//   len       - a length of the arrays;                
//                
   Procedure nspsbrCartToPolar(srcReal : pSingle;srcImag : pSingle;
                  mag : pSingle;          phase : pSingle;
                  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbrCartToPolar(srcReal : pdouble;srcImag : pdouble;
                  mag : pDouble;          phase : pDouble;
                  len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<c,z>bPolarToCart            
// DESCRIPTION:   
//   Convert polar coordinate to cartesian. Output data are formed as      
//            a complex vector.      
// PARAMETERS:    
//   mag       - an input vector containing the magnitude components;      
//   phase     - an input vector containing the phase components(in rad);  StdCall; external 'nsp.dll'; 
//   dst       - an output complex vector to store the cartesian coords;   
//   len       - a length of the arrays;                
//                
   Procedure nspcbPolarToCart(mag : pSingle; phase : pSingle;
                 dst : pSCplx;       len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbPolarToCart(mag : pDouble; phase : pDouble;
                 dst : pDCplx;       len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<s,d>brPolarToCart           
// DESCRIPTION:   
//   Convert polar coordinate to cartesian. Output data are formed as      
//            two real vectors.      
// PARAMETERS:    
//   mag       - an input vector containing the magnitude components;      
//   phase     - an input vector containing the phase components(in rad);  StdCall; external 'nsp.dll'; 
//   dstReal   - an output complex vector to store the coordinates X;      
//   dstImag   - an output complex vector to store the coordinates Y;      
//   len       - a length of the arrays;                
//                
   Procedure nspsbrPolarToCart(mag : pSingle; phase : pSingle;
                  dstReal : pSingle;   dstImag : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbrPolarToCart(mag : pDouble;phase : pDouble;
                  dstReal : pDouble;  dstImag : pDouble;len : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//  bMag, brMag, bPhase, brPhase     
//                
// Compute the magnitude and phase of complex vector elements.             
//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<c,z>bMag 
// DESCRIPTION:   
//   Compute the magnitude of complex vector elements.  
// PARAMETERS:    
//   src       - an input complex vector                
//   mag       - an output vector to store the magnitude components;       
//   len       - a length of the arrays;                
//                
   Procedure nspcbMag(src : pSCplx;  mag : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbMag(src : pDCplx;  mag : pDouble; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<s,d>brMag
// DESCRIPTION:   
//   Compute the magnitude of complex data formed as two real vectors.     
// PARAMETERS:    
//   srcReal   - an input vector containing a real part of complex data    
//   srcImag   - an input vector containing an imag. part of complex data  
//   mag       - an output vector to store the magnitude components;       
//   len       - a length of the arrays;                
//                
   Procedure nspsbrMag(srcReal : pSingle; srcImag : pSingle;
          mag : pSingle;           len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbrMag(srcReal : pdouble; srcImag : pdouble;
          mag : pDouble;           len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<c,z>bPhase                  
// DESCRIPTION:   
//   Compute the phase (in radians) of complex vector elements.            
// PARAMETERS:    
//   src       - an input complex vector                
//   phase     - an output vector to store the phase components;           
//   len       - a length of the arrays;                
//                
   Procedure nspcbPhase(src : pSCplx;  phase : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbPhase(src : pDCplx;  phase : pDouble; len : Integer);  StdCall; external 'nsp.dll';

// FUNCTION:      
//   nsp<s,d>brPhase                 
// DESCRIPTION:   
//   Compute the phase of complex data formed as two real vectors.         
// PARAMETERS:    
//   srcReal   - an input vector containing a real part of complex data    
//   srcImag   - an input vector containing an imag. part of complex data  
//   phase     - an output vector to store the magnitude components;       
//   len       - a length of the arrays;                
//
   Procedure nspsbrPhase(srcReal : pSingle; srcImag : pSingle;
            phase : pSingle;         len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbrPhase(srcReal : pdouble; srcImag : pdouble;
            phase : pDouble;         len : Integer);  StdCall; external 'nsp.dll';


   Function nspsDotProd(srcA : pSingle; srcB : pSingle; len : Integer): Single; StdCall; external 'nsp.dll';
   Function nspcDotProd(srcA : pSCplx; srcB : pSCplx; len : Integer): SCplx; StdCall; external 'nsp.dll';
   Function nspdDotProd(srcA : pDouble; srcB : pDouble; len : Integer): double; StdCall; external 'nsp.dll';
   Function nspzDotProd(srcA : pDCplx; srcB : pDCplx; len : Integer): DCplx; StdCall; external 'nsp.dll';

   Function nspwDotProd(srcA : pSmallInt; srcB : pSmallInt; len : integer;
                scaleMode : Integer;  var scaleFactor) :  SmallInt; StdCall; external 'nsp.dll';
   Function nspvDotProd(srcA : pWCplx; srcB : pWCplx; len : integer;
                scaleMode : Integer;  var scaleFactor): WCplx; StdCall; external 'nsp.dll';

   Function nspscDotProd(srcA : pSingle; srcB : pSCplx; len : Integer): SCplx; StdCall; external 'nsp.dll';
   Function nspcsDotProd(srcA : pSCplx; srcB : pSingle; len : Integer): SCplx; StdCall; external 'nsp.dll';
   Function nspdzDotProd(srcA : pDouble; srcB : pDCplx; len : Integer): DCplx; StdCall; external 'nsp.dll';
   Function nspzdDotProd(srcA : pDCplx; srcB : pDouble; len : Integer): DCplx; StdCall; external 'nsp.dll';

   Function nspwvDotProd(srcA : pSmallInt; srcB : pWCplx; len : integer;
                scaleMode : Integer;  var scaleFactor): WCplx; StdCall; external 'nsp.dll';
   Function nspvwDotProd(srcA : pWCplx; srcB : pSmallInt; len : integer;
                scaleMode : Integer;  var scaleFactor): WCplx; StdCall; external 'nsp.dll';

Type
  pNSPLibVersion = ^NSPLibVersion;
  NSPLibVersion = Record
    major : Integer;    // e.g. 3
    minor : Integer;    // e.g. 00
    build : Integer;    // e.g. 11
    Name  : pChar;      // e.g. "nspp6l.lib","nspm5.dll"
    Version : pChar;                 // e.g. "v3.00"
    InternalVersion : pChar;         // e.g. "[3.00.11, 07/12/96]"
    BuildDate : pChar;               // e.g. "Jun 1 96"
    CallConv : pChar;                // e.g. "DLL","Microsoft","Borland"
  End;

//--- NSPErrStatus,NSPErrMode Values Definition ---------------------------

Const
NSP_StsOk         =    0;         // everithing is ok
NSP_StsBackTrace  =   -1;         // pseudo error for back trace
NSP_StsError      =   -2;         // unknown /unspecified error
NSP_StsInternal   =   -3;         // internal error (bad state)
NSP_StsNoMem      =   -4;         // out of memory
NSP_StsBadArg     =   -5;         // function arg/param is bad
NSP_StsBadFunc    =   -6;         // unsupported function
NSP_StsNoConv     =   -7;         // iter. didn't converge
NSP_StsAutoTrace  =   -8;         // Tracing through nsptrace.h

NSP_ErrModeLeaf   =  0;           // Print error and exit program
NSP_ErrModeParent =  1;           // Print error and continue
NSP_ErrModeSilent =  2;           // Don't print and continue

Type
   NSPStatus = Integer;

   NSPErrorCallBack = Function (status : NSPStatus; funcname, context, filen : pChar;
                                line : Integer) : Integer; StdCall;
// -----------------------------------------------------------------------
// Flags for scaleMode parameter of nsp?Func(...,scaleMode,scaleFactor)
Const
NSP_NO_SCALE     =   $0000;      // ignore factor
NSP_FIXED_SCALE  =   $0002;      // fixed factor value
NSP_AUTO_SCALE   =   $0004;      // detect factor value

NSP_OVERFLOW     =   $0000;      // wrapround
NSP_SATURATE     =   $0001;      // saturate



//--- Get Library Version -------------------------------------------------
// Returns pointer to NSP lib info structure

   Function nspGetLibVersion : pNSPLibVersion;  StdCall; external 'nsp.dll';

//--- Get/Set ErrStatus ---------------------------------------------------

   Function nspGetErrStatus : NSPStatus;  StdCall; external 'nsp.dll';
   Procedure nspSetErrStatus(status : NSPStatus);  StdCall; external 'nsp.dll';

//--- NspStdErrMode Declaration -------------------------------------------

   Function nspGetErrMode : Integer;  StdCall; external 'nsp.dll';
   Procedure  nspSetErrMode(mode :Integer);  StdCall; external 'nsp.dll';

//--- nspError,nspErrorStr Declaration ------------------------------------

   Function spError(status : NSPStatus; func : pChar;
          context : pChar; filen : pchar; line : Integer) : NSPStatus;   StdCall; external 'nsp.dll';

   Function nspErrorStr(status : NSPStatus) : pChar;   StdCall; external 'nsp.dll';

//--- nspRedirectError Declaration ----------------------------------------

   Function nspNulDevReport(status : NSPStatus; funcname : pChar; 
               context : pChar; filen : pChar; line : Integer) : NSPStatus;  StdCall; external 'nsp.dll';

   Function nspStdErrReport(status : NSPStatus; funcname : pChar;
               context : pChar; filen : pChar; line : Integer) : NSPStatus;  StdCall; external 'nsp.dll';

   Function nspGuiBoxReport(status : NSPStatus; funcname : pChar;
               context : pChar; filen : pChar; line : Integer) : NSPStatus;  StdCall; external 'nsp.dll';


  Function nspRedirectError(nspErrorFunc : NSPErrorCallBack) : NSPErrorCallBack; StdCall; external 'nsp.dll';

Const
NSP_Forw        =      1;
NSP_Inv         =      2;
NSP_Init        =      4;
NSP_NoScale     =      8;
NSP_NoBitRev    =     16;
NSP_InBitRev    =     32;
NSP_OutBitRev   =     64;
NSP_OutRCPack   =    128;
NSP_OutRCPerm   =    256;
NSP_InRCPack    =    512;
NSP_InRCPerm    =   1024;
NSP_UseMMX      =   4096;
NSP_UseFastMMX  =   8192;
NSP_RADIX       =  16384;

//---------------------------------------------------------------------------
//
//        Dft
//
// Compute the forward  or inverse discrete Fourier transform  (DFT) of a
// signal.

   Procedure nspvDft(inSamps : pWCplx;  outSamps : pWCplx;
        len : Integer;  flags : Integer;
        scaleMode : Integer;var scaleFactor);  StdCall; external 'nsp.dll';
   Procedure nspcDft(inSamps : pSCplx;  outSamps : pSCplx;
        len : Integer;  flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzDft(inSamps : pDCplx;  outSamps : pDCplx;
        len : Integer;  flags : Integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        Fft       
//                  
// Compute  the forward  or inverse   fast Fourier  transform (FFT)  of a    
// signal.          

   Procedure nspvFft(samps : pWCplx; order : Integer; flags : Integer;
  scaleMode : Integer;        var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspcFft(samps : pSCplx; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzFft(samps : pDCplx; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspvFftNip(inSamps : pWCplx;  outSamps : pWCplx;
           order : Integer;   flags : Integer;
           scaleMode : Integer;var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure nspcFftNip(inSamps : pSCplx;  outSamps : pSCplx;
           order : Integer;   flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspzFftNip(inSamps : pDCplx;  outSamps : pDCplx;
           order : Integer;   flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspvrFft(reSamps : pSmallInt; imSamps : pSmallInt; order : Integer; flags : Integer;
   scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspcrFft(reSamps : pSingle; imSamps : pSingle; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspzrFft(reSamps : pDouble; imSamps : pDouble; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspvrFftNip(reInSamps : pSmallInt;  imInSamps : pSmallInt;
            reOutSamps : pSmallInt;      imOutSamps : pSmallInt;
            order : Integer; flags : Integer;
            scaleMode : Integer;        var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure nspcrFftNip(reInSamps : pSingle; imInSamps : pSingle;
            reOutSamps : pSingle;    imOutSamps : pSingle;
            order : Integer;          flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspzrFftNip(reInSamps : pDouble; imInSamps : pDouble;
             reOutSamps : pDouble;   imOutSamps : pDouble;
             order : Integer;          flags : Integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        RealFftl, RealFftlNip        
//                  
// Compute the  forward or inverse FFT  of a real signal  using RCPack or    
// RCPerm format.   

   Procedure nspwRealFftl(samps : pSmallInt; order : Integer; flags : Integer;
       scaleMode : Integer;        var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsRealFftl(samps : pSingle; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdRealFftl(samps : pdouble; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwRealFftlNip(inSamps : pSmallInt;  outSamps : pSmallInt;
                order : Integer;   flags : Integer;
                scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsRealFftlNip(inSamps : pSingle;  outSamps : pSingle;
                order : Integer;   flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdRealFftlNip(inSamps : pdouble;  outSamps : pdouble;
                order : Integer;   flags : Integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        CcsFftl, CcsFftlNip          
//                  
// Compute the  forward or inverse  FFT of a  complex conjugate-symmetric    
// (CCS) signal using RCPack or RCPerm format.            

   Procedure nspwCcsFftl(samps : pSmallInt;   order : Integer; flags : Integer;
      scaleMode : Integer;var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsCcsFftl(samps : pSingle;   order : Integer; flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdCcsFftl(samps : pdouble;   order : Integer; flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwCcsFftlNip(inSamps : pSmallInt;  outSamps : pSmallInt;
               order : Integer;   flags : Integer;
               scaleMode : Integer;var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsCcsFftlNip(inSamps : pSingle; outSamps : pSingle;
               order : Integer;  flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdCcsFftlNip(inSamps : pdouble; outSamps : pdouble;
               order : Integer;  flags : Integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        MpyRCPack2, MpyRCPack3       
//                  
// Multiply two vectors stored in RCPack format.

   Procedure nspwMpyRCPack2(src : pSmallInt;  dst : pSmallInt; order : Integer;
               scaleMode : Integer;         var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsMpyRCPack2(src : pSingle;dst : pSingle; order : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdMpyRCPack2(src : pdouble; dst : pDouble; order : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwMpyRCPack3(srcA : pSmallInt;srcB : pSmallInt;dst : pSmallInt;
               order : Integer;
               scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsMpyRCPack3(srcA : pSingle;srcB : pSingle; dst : pSingle;
               order : integer);  StdCall; external 'nsp.dll';

   Procedure nspdMpyRCPack3(var srcA, srcB : double;dst : pDouble;
               order : integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        MpyRCPerm2, MpyRCPerm3       
//                  
// Multiply two vectors stored in RCPerm format.          

   Procedure nspwMpyRCPerm2(src : pSmallInt; dst : pSmallInt; order : Integer;
               scaleMode : Integer;        var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsMpyRCPerm2(src : pSingle;dst : pSingle; order : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdMpyRCPerm2(src : pdouble; dst : pDouble; order : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwMpyRCPerm3(srcA : pSmallInt;srcB : pSmallInt;dst : pSmallInt;
               order : Integer;
               scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsMpyRCPerm3(srcA : pSingle;srcB : pSingle; dst : pSingle;
               order : integer);  StdCall; external 'nsp.dll';

   Procedure nspdMpyRCPerm3(var srcA, srcB : double;dst : pDouble;
               order : integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        RealFft, RealFftNip          
//                  
// Compute the forward or inverse FFT of a real signal.   

   Procedure nspwRealFft(samps : pSmallInt; order : Integer;  flags : Integer;
      scaleMode : Integer;         var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsRealFft(samps : pSingle; order : Integer;  flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdRealFft(samps : pdouble; order : Integer;  flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwRealFftNip(inSamps : pSmallInt;  outSamps : pWCplx;
               order : Integer;    flags : Integer;
               scaleMode : Integer;var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsRealFftNip(inSamps : pSingle;  outSamps : pSCplx;
               order : Integer;   flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdRealFftNip(inSamps : pdouble;  outSamps : pDCplx;
               order : Integer;   flags : Integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        CcsFft, CcsFftNip            
//                  
// Compute the  forward or inverse  FFT of a  complex conjugate-symmetric    
// (CCS) signal.    

   Procedure nspwCcsFft(samps : pSmallInt; order : Integer; flags : Integer;
     scaleMode : Integer;        var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsCcsFft(samps : pSingle; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdCcsFft(samps : pdouble; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwCcsFftNip(inSamps : pWCplx;  outSamps : pSmallInt;
              order : Integer;    flags : Integer;
              scaleMode : Integer;var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsCcsFftNip(inSamps : pSCplx; outSamps : pSingle;
              order : Integer;  flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdCcsFftNip(inSamps : pDCplx; outSamps : pdouble;
              order : Integer;  flags : Integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        nsp?Real2Fft, nsp?Real2FftNip
//                  
// Compute the forward or inverse FFT of two real signals.

   Procedure nspwReal2Fft(xSamps : pSmallInt; ySamps : pSmallInt; order : Integer; flags : Integer;
       scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsReal2Fft(xSamps : pSingle; ySamps : pSingle; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdReal2Fft(xSamps : pDouble; ySamps : pDouble; order : Integer; flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwReal2FftNip(xInSamps : pSmallInt; xOutSamps : pWCplx;
          yInSamps : pSmallint;yOutSamps : pWCplx; order : Integer;
            flags : Integer;
                scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsReal2FftNip(xInSamps : pSingle; xOutSamps : pSCplx;
          yInSamps : pSingle; yOutSamps : pSCplx; order : Integer;
            flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdReal2FftNip(xInSamps : pDouble; xOutSamps : pDCplx;
          yInSamps : pDouble; yOutSamps : pDCplx; order : Integer;
            flags : Integer);  StdCall; external 'nsp.dll';

//---------------------------------------------------------------------------
//        nsp?Ccs2Fft, nsp?Ccs2FftNip  
//                  
// Compute the forward or reverse  FFT of two complex conjugate-symmetric    
// (CCS) signals.   

   Procedure nspwCcs2Fft(xSamps : pSmallInt; ySamps : pSmallInt; order : Integer;flags : Integer;
      scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspsCcs2Fft(xSamps : pSingle; ySamps : pSingle; order : Integer;flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdCcs2Fft(xSamps : pDouble; ySamps : pDouble; order : Integer;flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspwCcs2FftNip(xInSamps : pWCplx; xOutSamps : pSmallInt;
         yInSamps : pWCplx; yOutSamps : pSmallint; order : Integer;
           flags : Integer;
               scaleMode : Integer; var scaleFactor : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsCcs2FftNip(xInSamps : pSCplx; xOutSamps : pSingle;
         yInSamps : pSCplx; yOutSamps : pSingle; order : Integer;
           flags : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdCcs2FftNip(xInSamps : pDCplx; xOutSamps : pdouble;
         yInSamps : pDCplx; yOutSamps : pDouble; order : Integer;
           flags : Integer);  StdCall; external 'nsp.dll';


   Procedure nspsFilter2D(x : pSingle; xCols, xRows : Integer;
       h : pSingle; hCols, hRows : Integer;
             y : pSingle);  StdCall; external 'nsp.dll';

   Procedure nspdFilter2D(x : pdouble; xCols, xRows : Integer;
       h : pdouble; hCols, hRows : Integer;
             y : pdouble);  StdCall; external 'nsp.dll';

   Procedure nspwFilter2D(x : pSmallInt; xCols, xRows : Integer;
       h : pSmallInt; hCols, hRows : Integer;
             y : pSmallInt;
       scaleMode : Integer; var scaleFactorPtr : Integer);  StdCall; external 'nsp.dll';



//=== FIR high-level ======================================================

Type

   pFirSect = ^firSect;
   firSect = Record
      num : Integer;          // section number
      len : Integer;          // section taps length in use
      xId : Integer;          // section input data index
      tId : Integer;          // section taps index
      // ---- fields below are special for w-flavor -------------------------
      initlen : Integer;      // section taps length
      tapsGate : Array[0..3] of Pointer;// shifted section taps pointers
   End;

    NSPFirState = Record
      upFactor : Integer;  // up
      upPhase : Integer;   //  parameters
      downFactor : Integer;// down
      downPhase : Integer; //  parameters
      isMultiRate : Integer;// multi-rate mode flag
      isInit : Pointer;     // init flag

      tapsLen : Integer;  // taps length in use
      taps : Pointer;        // taps pointer in use
      tapsBlk : Integer;

      dlylLen : Integer;  // delay line length
      dlyl : Pointer;        // delay line pointer
      dlylLimit : Pointer;  // delay line buffer end pointer
      useDlyl : Pointer;    // delay line pointer in use
      inpLen : Integer;     // input buffer length
      useInp : Integer;     // input buffer length used

      sect : pFirSect;   // FIR sections

      // ---- fields below are special for w-flavor -------------------------
      tapsInitLen : Integer;// init taps length
      tapsGate : Array [0..3] of Pointer;// shifted taps addresses
  End;

//-------------------------------------------------------------------------
// nsp?FirInit(), nsp?FirInitMr()
// nspFirFree()
//
// Initialize a finite impulse response high-level (FIRH) filter

   Procedure nspsFirInit(tapVals : pSingle; tapsLen : Integer;
       dlyVals : pSingle; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspcFirInit(tapVals : pSCplx; tapsLen : Integer;
       dlyVals : pSCplx; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspscFirInit(tapVals : pSCplx; tapsLen : Integer;
       dlyVals : pSingle; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspcsFirInit(tapVals : pSingle; tapsLen : Integer;
       dlyVals : pSCplx; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspdFirInit(tapVals : pDouble; tapsLen : Integer;
       dlyVals : pDouble; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspzFirInit(tapVals : pDCplx; tapsLen : Integer;
       dlyVals : pDCplx; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspdzFirInit(tapVals : pDCplx; tapsLen : Integer;
       dlyVals : pDouble; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspzdFirInit(tapVals : pDouble; tapsLen : Integer;
       dlyVals : pDCplx; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspwFirInit(tapVals : pSmallint;  tapsLen : Integer;
       dlyVals : pSmallint;  var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspsFirInitMr(tapVals : pSingle;  tapsLen : Integer;
         dlyVals : pSingle;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspcFirInitMr(tapVals : pSCplx;  tapsLen : Integer;
         dlyVals : pSCplx;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspscFirInitMr(tapVals : pSCplx;  tapsLen : Integer;
         dlyVals : pSingle;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspcsFirInitMr(tapVals : pSingle;  tapsLen : Integer;
         dlyVals : pSCplx;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspdFirInitMr(tapVals : pDouble;  tapsLen : Integer;
         dlyVals : pDouble;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspzFirInitMr(tapVals : pDCplx;  tapsLen : Integer;
         dlyVals : pDCplx;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspdzFirInitMr(tapVals : pDCplx;  tapsLen : Integer;
         dlyVals : pDouble;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspzdFirInitMr(tapVals : pDouble;  tapsLen : Integer;
         dlyVals : pDCplx;   upFactor, upPhase : Integer;
               downFactor, downPhase : Integer;
         var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspwFirInitMr(tapVals : pSmallint;tapsLen : Integer;
        dlyVals : pSmallint; upFactor, upPhase : Integer;
        downFactor, downPhase : Integer;
        var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspFirFree(var statePtr : NSPFirState);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
// nsp?Fir(), nsp?bFir()             
//
// Dot and block product FIRH filtering

   Function nspsFir(var statePtr : NSPFirState; samp : Single): Single; StdCall; external 'nsp.dll';
   Function nspcFir(var statePtr : NSPFirState; samp : SCplx) : SCplx; StdCall; external 'nsp.dll';
   Function nspscFir(var statePtr : NSPFirState; samp : Single) : SCplx; StdCall; external 'nsp.dll';
   Function nspcsFir(var statePtr : NSPFirState; samp : SCplx) : SCplx; StdCall; external 'nsp.dll';

   Function nspdFir(var statePtr : NSPFirState; samp : double): double; StdCall; external 'nsp.dll';
   Function nspzFir(var statePtr : NSPFirState; samp : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspdzFir(var statePtr : NSPFirState; samp : double): DCplx; StdCall; external 'nsp.dll';
   Function nspzdFir(var statePtr : NSPFirState; samp : DCplx): DCplx; StdCall; external 'nsp.dll';

   Function nspwFir(var statePtr : NSPFirState; samp : SmallInt;
    sMode : Integer; var sFctPtr : Integer) :  SmallInt; StdCall; external 'nsp.dll';

   Procedure nspsbFir(var statePtr : NSPFirState; inSamps : pSingle;
    outSamps : pSingle;    numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbFir(var statePtr : NSPFirState; inSamps : pSCplx;
    outSamps : pSCplx;    numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspscbFir(var statePtr : NSPFirState; inSamps : pSingle;
    outSamps : pSCplx;    numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcsbFir(var statePtr : NSPFirState; inSamps : pSCplx;
    outSamps : pSCplx;    numIters : Integer);  StdCall; external 'nsp.dll';

   Procedure nspdbFir(var statePtr : NSPFirState; inSamps : pdouble;
    outSamps : pdouble;    numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbFir(var statePtr : NSPFirState; inSamps : pDCplx;
    outSamps : pDCplx;    numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdzbFir(var statePtr : NSPFirState; inSamps : pdouble;
    outSamps : pDCplx;    numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzdbFir(var statePtr : NSPFirState; inSamps : pDCplx;
    outSamps : pDCplx;    numIters : Integer);  StdCall; external 'nsp.dll';

   Procedure nspwbFir(var statePtr : NSPFirState; inSamps : pSmallInt;
    outSamps : pSmallInt;     numIters : Integer;
    sMode : Integer; var sFctPtr : Integer);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
// nsp?FirGetTaps(), nsp?FirSetTaps()
//                
// Utility functions to get and set the FIR taps coefficients              

   Procedure nspsFirGetTaps(var statePtr : NSPFirState; outTaps : pSingle);  StdCall; external 'nsp.dll';
   Procedure nspcFirGetTaps(var statePtr : NSPFirState; outTaps : pSCplx);  StdCall; external 'nsp.dll';
   Procedure nspdFirGetTaps(var statePtr : NSPFirState; outTaps : pDouble);  StdCall; external 'nsp.dll';
   Procedure nspzFirGetTaps(var statePtr : NSPFirState; outTaps : pDCplx);  StdCall; external 'nsp.dll';

   Procedure nspwFirGetTaps(var statePtr : NSPFirState; outTaps : pSmallInt);  StdCall; external 'nsp.dll';

   Procedure nspsFirSetTaps(inTaps : pSingle; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspcFirSetTaps(inTaps : pSCplx; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspdFirSetTaps(inTaps : pDouble;var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspzFirSetTaps(inTaps : pDCplx;var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspwFirSetTaps(inTaps : pSmallInt; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
// nsp?FirGetDlyl(), nsp?FirSetDlyl()
//
// Utility functions to get and set the FIR delay line contents            

   Procedure nspsFirGetDlyl(var statePtr : NSPFirState; outDlyl : pSingle);  StdCall; external 'nsp.dll';
   Procedure nspcFirGetDlyl(var statePtr : NSPFirState; outDlyl : pSCplx);  StdCall; external 'nsp.dll';
   Procedure nspdFirGetDlyl(var statePtr : NSPFirState; outDlyl : pDouble);  StdCall; external 'nsp.dll';
   Procedure nspzFirGetDlyl(var statePtr : NSPFirState; outDlyl : pDCplx);  StdCall; external 'nsp.dll';

   Procedure nspwFirGetDlyl(var statePtr : NSPFirState; outDlyl : pSmallInt);  StdCall; external 'nsp.dll';

   Procedure nspsFirSetDlyl(inDlyl : pSingle;var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspcFirSetDlyl(inDlyl : pSCplx; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspdFirSetDlyl(inDlyl : pDouble; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';
   Procedure nspzFirSetDlyl(inDlyl : pDCplx; var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

   Procedure nspwFirSetDlyl(inDlyl : pSmallInt;var statePtr : NSPFirState);  StdCall; external 'nsp.dll';

// --- FIR low level filter structure -------------------------------------

// --- Finite impulse response filter -------------------------------------

Type
  NSPFirTapState  = Record
   taps : Pointer;
   len : Integer;
   upFactor : Integer;
   upPhase : Integer;
   downFactor : Integer;
   downPhase : Integer;
   isMultiRate : Integer;
   isFilterMode : Integer;

  End;

// --- Delay line of FIR low level filter ---------------------------------

  NSPFirDlyState  = Record
   dlyl : Pointer;
   len : Integer;
   isFilterMode  : Integer;
  End;

// ------------------------------------------------------------------------
//        FirlInit, FirlInitMr, FirlInitDlyl
//
// Low level functions for cyclic FIR filtering via a tapped delay line.

   Procedure nspsFirlInit(taps : pSingle; tapsLen : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspdFirlInit(taps : pDouble; tapsLen : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspcFirlInit(taps : pSCplx; tapsLen : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspzFirlInit(taps : pDCplx; tapsLen : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspwFirlInit(taps : pSmallInt; tapsLen : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';

   Procedure nspsFirlInitMr(taps : pSingle;tapsLen : Integer; upFactor, upPhase : Integer;
 downFactor, downPhase : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspdFirlInitMr(taps : pDouble; tapsLen : Integer; upFactor, upPhase : Integer;
 downFactor, downPhase : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspcFirlInitMr(taps : pSCplx; tapsLen : Integer; upFactor, upPhase : Integer;
 downFactor, downPhase : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspzFirlInitMr(taps : pDCplx; tapsLen : Integer; upFactor, upPhase : Integer;
 downFactor, downPhase : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspwFirlInitMr(taps : pSmallInt; tapsLen : Integer; upFactor, upPhase : Integer;
 downFactor, downPhase : Integer; tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';

   Procedure nspsFirlInitDlyl(var tapStPtr : NSPFirTapState; dlyl : psingle;
                 var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspcFirlInitDlyl( var tapStPtr : NSPFirTapState; dlyl : pScplx;
                 var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspdFirlInitDlyl(var tapStPtr : NSPFirTapState; dlyl : pdouble;
                 var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspzFirlInitDlyl(var tapStPtr : NSPFirTapState; dlyl : pDCplx;
                 var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspwFirlInitDlyl(var tapStPtr : NSPFirTapState; dlyl : pSmallint;
                 var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        Firl, bFirl
//
// Filter either a single sample or  block of samples through a low-level
// FIR filter.

   Function nspsFirl(var tapStPtr : NSPFirTapState;
           var dlyStPtr : NSPFirDlyState; samp : Single): Single; StdCall; external 'nsp.dll';
   Function nspcFirl(var tapStPtr : NSPFirTapState;
           var dlyStPtr : NSPFirDlyState; samp : SCplx) : SCplx; StdCall; external 'nsp.dll';
   Function nspdFirl(var tapStPtr : NSPFirTapState;
           var dlyStPtr : NSPFirDlyState; samp : double): double; StdCall; external 'nsp.dll';
   Function nspzFirl(var tapStPtr : NSPFirTapState;
           var dlyStPtr : NSPFirDlyState; samp : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspwFirl(var tapStPtr : NSPFirTapState;
           var dlyStPtr : NSPFirDlyState; samp : pSmallInt;
           scaleMode : Integer; var scaleFactorPtr : Integer) :  SmallInt; StdCall; external 'nsp.dll';

   Function nspscFirl(var tapStPtr : NSPFirTapState;
            var dlyStPtr : NSPFirDlyState; samp : Single): SCplx; StdCall; external 'nsp.dll';
   Function nspcsFirl(var tapStPtr : NSPFirTapState;
            var dlyStPtr : NSPFirDlyState; samp : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzFirl(var tapStPtr : NSPFirTapState;
            var dlyStPtr : NSPFirDlyState; samp : double): DCplx; StdCall; external 'nsp.dll';
   Function nspzdFirl(var tapStPtr : NSPFirTapState;
            var dlyStPtr : NSPFirDlyState; samp : DCplx): DCplx; StdCall; external 'nsp.dll';

   Procedure   nspsbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
      inSamps : pSingle; outSamps : pSingle; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspcbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
      inSamps : pSCplx; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspdbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
      inSamps : pdouble; outSamps : pdouble; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspzbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
      inSamps : pDCplx; outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspwbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
      inSamps : pSmallInt; outSamps : pSmallInt; numIters : Integer;
        scaleMode : Integer; var scaleFactorPtr : Integer);  StdCall; external 'nsp.dll';

   Procedure   nspscbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
       inSamps : pSingle; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspcsbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
       inSamps : pSCplx; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspdzbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
       inSamps : pdouble; outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure   nspzdbFirl(var tapStPtr : NSPFirTapState; var dlyStPtr : NSPFirDlyState;
       inSamps : pDCplx; outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        FirlGetTaps, FirlSetTaps   
//                
// Utility functions to get and set the tap coefficients of low-level FIR  
// filters.       

   Procedure nspsFirlGetTaps(var tapStPtr : NSPFirTapState; outTaps : pSingle);  StdCall; external 'nsp.dll';
   Procedure nspcFirlGetTaps(var tapStPtr : NSPFirTapState; outTaps : pSCplx);  StdCall; external 'nsp.dll';
   Procedure nspdFirlGetTaps(var tapStPtr : NSPFirTapState; outTaps : pDouble);  StdCall; external 'nsp.dll';
   Procedure nspzFirlGetTaps(var tapStPtr : NSPFirTapState; outTaps : pDCplx);  StdCall; external 'nsp.dll';
   Procedure nspwFirlGetTaps(var tapStPtr : NSPFirTapState; outTaps : pSmallInt);  StdCall; external 'nsp.dll';

   Procedure nspsFirlSetTaps(inTaps : pSingle; Var tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspcFirlSetTaps(inTaps : pSCplx;  Var tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspdFirlSetTaps(inTaps : pDouble; Var tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspzFirlSetTaps(inTaps : pDCplx; Var tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';
   Procedure nspwFirlSetTaps(inTaps : pSmallInt;  Var tapStPt : NSPFirTapState);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        FirlGetDlyl, FirlSetDlyl   
//                
// Utility functions to get and set  the delay line contents of low-level  
// FIR filters.   

   Procedure nspsFirlGetDlyl(var tapStPtr : NSPFirTapState;
          var dlyStPtr : NSPFirDlyState; outDlyl : pSingle);  StdCall; external 'nsp.dll';
   Procedure nspcFirlGetDlyl(var tapStPtr : NSPFirTapState;
          var dlyStPtr : NSPFirDlyState; outDlyl : pSCplx);  StdCall; external 'nsp.dll';
   Procedure nspdFirlGetDlyl(var tapStPtr : NSPFirTapState;
          var dlyStPtr : NSPFirDlyState; outDlyl : pDouble);  StdCall; external 'nsp.dll';
   Procedure nspzFirlGetDlyl(var tapStPtr : NSPFirTapState;
          var dlyStPtr : NSPFirDlyState; outDlyl : pDCplx);  StdCall; external 'nsp.dll';
   Procedure nspwFirlGetDlyl(var tapStPtr : NSPFirTapState;
          var dlyStPtr : NSPFirDlyState; outDlyl : pSmallInt);  StdCall; external 'nsp.dll';


   Procedure nspsFirlSetDlyl(var tapStPtr : NSPFirTapState; inDlyl : pSingle;
                var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspcFirlSetDlyl(var tapStPtr : NSPFirTapState; inDlyl : pSCplx; 
                var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspdFirlSetDlyl(var tapStPtr : NSPFirTapState; inDlyl : pDouble; 
                var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspzFirlSetDlyl(var tapStPtr : NSPFirTapState; inDlyl : pDCplx;
                var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';
   Procedure nspwFirlSetDlyl(var tapStPtr : NSPFirTapState; inDlyl : pSmallInt;
                var dlyStPtr : NSPFirDlyState);  StdCall; external 'nsp.dll';

// Single Frequency DFT structures.

Type
  NSPSGoertzState = Record
    freq : Single;
    cosV : single;
    sinV : single;
    cpcV : single;
    prev2re : single;
    prev2im : single;
    prev1re : single;
    prev1im : single;
  End;

  NSPCGoertzState = NSPSGoertzState;

  NSPDGoertzState = Record
    freq : double;
    cosV : double;
    sinV : double;
    cpcV : double;
    prev2re : double;
    prev2im : double;
    prev1re : double;
    prev1im : double;
  End;

 NSPZGoertzState = NSPDGoertzState;

// ------------------------------------------------------------------------
//        GoertzInit                 
//                
// Initializes the coefficients and zeros the delay line.                  

   Procedure nspsGoertzInit(freq : Single; var stPtr : NSPSGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspcGoertzInit(freq : Single; var stPtr : NSPCGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspdGoertzInit(freq : double; var stPtr : NSPDGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspzGoertzInit(freq : double;  var stPtr : NSPZGoertzState);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        GoertzReset                
//                
// Zeros the delay line.

   Procedure nspsGoertzReset(var stPtr : NSPSGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspcGoertzReset(var stPtr : NSPCGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspdGoertzReset(var stPtr : NSPDGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspzGoertzReset(var stPtr : NSPZGoertzState);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        Goertz  
//
// Single Frequency DFT (Goertzel algorithm)            

   Function nspsGoertz(var stPtr : NSPSGoertzState; sample : Single): SCplx; StdCall; external 'nsp.dll';
   Function nspcGoertz(var stPtr : NSPCGoertzState; sample : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdGoertz(var stPtr : NSPDGoertzState; sample : double): DCplx; StdCall; external 'nsp.dll';
   Function nspzGoertz(var stPtr : NSPZGoertzState; sample : DCplx): DCplx; StdCall; external 'nsp.dll';

   Function nspsbGoertz(var stPtr : NSPSGoertzState; inSamps : pSingle;len : Integer): SCplx; StdCall; external 'nsp.dll';
   Function nspcbGoertz(var stPtr : NSPCGoertzState; inSamps : pSCplx;len : Integer): SCplx; StdCall; external 'nsp.dll';
   Function nspdbGoertz(var stPtr : NSPDGoertzState; inSamps : pdouble;len : Integer): DCplx; StdCall; external 'nsp.dll';
   Function nspzbGoertz(var stPtr : NSPZGoertzState; inSamps : pDCplx;len : Integer): DCplx; StdCall; external 'nsp.dll';

Type
  NSPWGoertzState = NSPSGoertzState;
  NSPVGoertzState = NSPSGoertzState;

// ------------------------------------------------------------------------
//        GoertzInit                 
//                
// Initializes the coefficients and zeros the delay line.                  

   Procedure nspwGoertzInit(freq : Single; var stPtr : NSPWGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspvGoertzInit(freq : Single; var stPtr : NSPVGoertzState);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        GoertzReset                
//                
// Zeros the delay line.             

   Procedure nspwGoertzReset(var stPtr : NSPWGoertzState);  StdCall; external 'nsp.dll';
   Procedure nspvGoertzReset(var stPtr : NSPVGoertzState);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        Goertz  
//                
// Single Frequency DFT (Goertzel algorithm)            

   Function nspwGoertz(var stPtr : NSPWGoertzState; sample : smallint;
      scaleMode : Integer;  var scaleFactor): WCplx; StdCall; external 'nsp.dll';
   Function nspvGoertz(var stPtr : NSPVGoertzState; sample : WCplx;
      scaleMode : Integer;  var scaleFactor): WCplx; StdCall; external 'nsp.dll';

   Function nspwbGoertz(var stPtr : NSPWGoertzState; inSamps : pSmallInt;
       len : integer; scaleMode : Integer;  var scaleFactor): WCplx; StdCall; external 'nsp.dll';
   Function nspvbGoertz(var stPtr : NSPVGoertzState; inSamps : pWCplx;
       len : integer; scaleMode : Integer;  var scaleFactor): WCplx; StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//         IirInit, IirInitBq, IirFree
//
// These functions initialize an IIR filter and provide a higher-level
// interface than the corresponding low-level IIR functions.
//

//-------------------------------------------------------------------------
// FUNCTION:
//   nsp<s,c,d,z>IirInit
//   nsp<sc,cs,dz,zd>IirInit
// DESCRIPTION:
//   These functions initialize an arbitrary order IIR filter.
// PARAMETERS:
//   iirType  - input, describes the filter structure to use, and currently
//              must be NSP_IirDefault;
//   tapVals  - input, the 2*(order+1) length array specifies the filter
//              coefficients as discussed for the low-level IIR function
//              nsp?IirlInit();  StdCall; external 'nsp.dll';
//   order    - input, sets polinomial order;
//   statePtr - output, the pointer to the IIR filter state structure.
//
//---- User tap type ------------------------------------------------------

Type
  NSPIirType = (NSP_IirNull, NSP_IirDefault);

//---- Implementation tap type of initialization --------------------------

  NSPIirInit = (NSP_IirInit, NSP_IirInitBq);

//---- IIR filter tap state structure -------------------------------------

  NSPIirTapState = Record
    order : Integer;                 //  order/number of biquads
    taps  : Pointer;                 //  pointer to taps
    iirtype : NSPIirType;            //  user type
    initype : NSPIirInit;            //  type of initialiuzation
  End;

//---- IIR filter delay line state structure ------------------------------

  NSPIirDlyState = Record
    dlyl : Pointer;                  //  pointer to delay line
  End;

//---- IIR filter state structure -----------------------------------------

  NSPIirState = Record
    tapSt : NSPIirTapState;              // tap state structure
    dlySt : NSPIirDlyState;              // delay line state structure
    init : Integer;               // initialization flag
  End;

   Procedure nspsIirInit(iirType : NSPIirType; tapVals : single;
      order : Integer;         var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspcIirInit(iirType : NSPIirType; tapVals : pSCplx;
      order : Integer;         var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspdIirInit(iirType : NSPIirType; tapVals : pDouble;
      order : Integer;         var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspzIirInit(iirType : NSPIirType; tapVals : pDCplx;
      order : Integer;        var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspwIirInit(iirType : NSPIirType; tapVals : pSmallInt;
      order : Integer;        var statePtr : NSPIirState);  StdCall; external 'nsp.dll';

   Procedure nspscIirInit(iirType : NSPIirType; tapVals : pSCplx;
       order : Integer;        var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspcsIirInit(iirType : NSPIirType; tapVals : single;
       order : Integer;        var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspdzIirInit(iirType : NSPIirType; tapVals : pDCplx;
       order : Integer;        var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspzdIirInit(iirType : NSPIirType; tapVals : pDouble;
       order : Integer;        var statePtr : NSPIirState);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,c,d,z>IirInitBq           
//   nsp<sc,cs,dz,zd>IirInitBq       
// DESCRIPTION:   
//   These functions initialize an IIR filter defined by a cascade of      
//   biquads.     
// PARAMETERS:    
//   iirType  - input, describes the filter structure to use, and currently
//              must be NSP_IirDefault;                 
//   tapVals  - input, the 6*numQuads length array specifies the filter    
//              coefficients as discussed for the low-level IIR function   
//              nsp?IirlInitBq();  StdCall; external 'nsp.dll';   
//   numQuads - input, sets the number of biduads;      
//   var tapStPtr - output, pointer to the IIR state structure.                
//                

   Procedure nspsIirInitBq(iirType : NSPIirType; tapVals : single;
        numQuads : Integer;     var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspcIirInitBq(iirType : NSPIirType; tapVals : pSCplx;
        numQuads : Integer;     var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspdIirInitBq(iirType : NSPIirType; tapVals : pDouble;
        numQuads : Integer;     var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspzIirInitBq(iirType : NSPIirType; tapVals : pDCplx;
        numQuads : Integer;     var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspwIirInitBq(iirType : NSPIirType; tapVals : pSmallInt;
        numQuads : Integer;     var statePtr : NSPIirState);  StdCall; external 'nsp.dll';

   Procedure nspscIirInitBq(iirType : NSPIirType; tapVals : pSCplx;
         numQuads : Integer;      var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspcsIirInitBq(iirType : NSPIirType; tapVals : single;
         numQuads : Integer;      var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspdzIirInitBq(iirType : NSPIirType; tapVals : pDCplx;
         numQuads : Integer;      var statePtr : NSPIirState);  StdCall; external 'nsp.dll';
   Procedure nspzdIirInitBq(iirType : NSPIirType; tapVals : pDouble;
         numQuads : Integer;      var statePtr : NSPIirState);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
// FUNCTION:      
//   nspIirFree   
// DESCRIPTION:   
//   This function must be called after all filtering is done to free      
//   dynamic memory associated with statePtr. After calling this function,
//   statePtr should not be referenced again.
// PARAMETERS:
//   statePtr - pointer to the IIR filter state structure.
//

   Procedure nspIirFree(var statePtr : NSPIirState);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//         Iir, bIir
//
// Filter a signal through a IIR filter.
//

//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,c,d,z>Iir                 
//   nsp<sc,cs,dz,zd>Iir             
// DESCRIPTION:   
//   These functions filter a single sample samp through an IIR filter and
//   return the result.              
// PARAMETERS:    
//   statePtr - input, pointer to the IIR state structure;                 
//   samp     - input, the sample value.                
// RETURNS:       
//   the filtering result.           
//                

   Function nspsIir(var statePtr : NSPIirState; samp : Single): Single; StdCall; external 'nsp.dll';
   Function nspcIir(var statePtr : NSPIirState; samp : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdIir(var statePtr : NSPIirState; samp : double): double; StdCall; external 'nsp.dll';
   Function nspzIir(var statePtr : NSPIirState; samp : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspwIir(var statePtr : NSPIirState; samp : pSmallInt;
    scaleMode : Integer;var scaleFactor : Integer) :  SmallInt; StdCall; external 'nsp.dll';

   Function nspscIir(var statePtr : NSPIirState; samp : Single): SCplx; StdCall; external 'nsp.dll';
   Function nspcsIir(var statePtr : NSPIirState; samp : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzIir(var statePtr : NSPIirState; samp : double): DCplx; StdCall; external 'nsp.dll';
   Function nspzdIir(var statePtr : NSPIirState; samp : DCplx): DCplx; StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,c,d,z>bIir                
//   nsp<sc,cs,dz,zd>bIir            
// DESCRIPTION:   
//   These functions filter a block of numIters samples inSamps through an 
//   IIR filter and return the result in outSamps.      
// PARAMETERS:    
//   statePtr - input, pointer to the IIR state structure;                 
//   inSamps  - input, the pointer to block of numIters samples;           
//   outSamps - output, the pointer to block of numIters filtered samples; 
//   numIters - input, the size of samples' block.      
//                

   Procedure nspsbIir(var statePtr : NSPIirState;  inSamps : pSingle;
   outSamps : pSingle;      numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbIir(var statePtr : NSPIirState;  inSamps : pSCplx;
   outSamps : pSCplx;      numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbIir(var statePtr : NSPIirState;  inSamps : pdouble;
   outSamps : pdouble;      numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbIir(var statePtr : NSPIirState;  inSamps : pDCplx;
   outSamps : pDCplx;      numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbIir(var statePtr : NSPIirState;  inSamps : pSmallInt;
   outSamps : pSmallInt;       numIters : Integer;
   scaleMode : Integer;       var scaleFactor);  StdCall; external 'nsp.dll';

   Procedure nspscbIir(var statePtr : NSPIirState; inSamps : pSingle;
    outSamps : pSCplx;     numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcsbIir(var statePtr : NSPIirState; inSamps : pSCplx;
    outSamps : pSCplx;     numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdzbIir(var statePtr : NSPIirState; inSamps : pdouble;
    outSamps : pDCplx;     numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzdbIir(var statePtr : NSPIirState; inSamps : pDCplx;
    outSamps : pDCplx;     numIters : Integer);  StdCall; external 'nsp.dll';


//-------------------------------------------------------------------------
//         IirlInit, IirlInitBq, IirlInitDlyl
//
// Perform infinite impulse response filtering.
//-------------------------------------------------------------------------
// FUNCTION:
//   nsp<s,c,d,z>IirlInit
// DESCRIPTION:
//   Taps initialization of low-level arbitrary order IIR-filter with
//   transfer function expressed as the ratio of two polinomials of order
//   "order".
// PARAMETERS:    
//   iirType  - input, determines the filter structure to use, currently   
//              must be NSP_IirDefault;                 
//   taps     - input/output, sets the array taps of filter with transfer  
//              function             
//B0+B1/z+B2/z/z ... +Border/z**order
//              H(z)=------------------------------------                  
//A0+A1/z+A2/z/z ... +Aorder/z**order
//              in following order:  
//                  B0, B1, ... Border,A0, A1, ... Aorder.                 
//              A0 must not be zero; 
//   order    - input, sets polinomial order;           
//   var tapStPtr - output, the pointer to the tap state structure.            
//                

   Procedure nspwIirlInit(iirType : NSPIirType; taps : pSmallInt;  order : Integer;
       var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspsIirlInit(iirType : NSPIirType; taps : pSingle; order : Integer;
       var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspcIirlInit(iirType : NSPIirType; taps : pSCplx;  order : Integer;
       var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspdIirlInit(iirType : NSPIirType; taps : pDouble;  order : Integer;
       var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspzIirlInit(iirType : NSPIirType; taps : pDCplx;  order : Integer;
       var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,c,d,z>IirlInitBq          
// DESCRIPTION:   
//   Low-level IIR-filter iinitialization taps to reference a cascade of   
//   biquads. Transfer function is a product of numQuads biquads.          
// PARAMETERS:    
//   iirType  - input, determines the filter structure to use, currently   
//              must be NSP_IirDefault;                 
//   taps     - input/output, sets the array taps of filter                
//              with the transfer function              
//(B10+B11/z+B12/z**2) *... *(BnQ0+BnQ1/z+BnQ2/z**2)    
//              H(z)=--------------------------------------------------    
//(A10+A11/z+A12/z**2) *... *(AnQ0+AnQ1/z+AnQ2/z**2)    
//              in following order:  
//B10, B11, B12, A10, A11, A12, ... ,
//BnQ0, BnQ1, BnQ2, AnQ0, AnQ1, AnQ2.
//              All Bi0 and Ai0 must not be zero;       
//   numQuads - input, sets the number of biduads;      
//   var tapStPtr - output, pointer to the tap state structure.                
//                

   Procedure nspwIirlInitBq(iirType : NSPIirType; taps : pSmallInt; numQuads : Integer;
         var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspsIirlInitBq(iirType : NSPIirType; taps : pSingle;numQuads : Integer;
         var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspcIirlInitBq(iirType : NSPIirType; taps : pSCplx; numQuads : Integer;
         var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspdIirlInitBq(iirType : NSPIirType; taps : pDouble; numQuads : Integer;
         var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';
   Procedure nspzIirlInitBq(iirType : NSPIirType; taps : pDCplx; numQuads : Integer;
         var tapStPtr : NSPIirTapState  );  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,c,d,z>IirlInitDlyl        
// DESCRIPTION:   
//   Initialization of delay array for low-level IIR-filter.               
// PARAMETERS:    
//   var tapStPtr - input, the pointer to the tap state structure; 
//              initializated previously by the function nsp?IirInit or    
//              nsp?IirInitBq;       
//   dlyl     - input/output, the delay line array. In this implementation 
//              the input array element values are ignored.                
//   var dlyStPtr - output, the pointer to the delay line state structure.     
//                

   Procedure nspwIirlInitDlyl(var tapStPtr : NSPIirTapState; dlyl : pSmallint;
                 var dllyStPtr : NSPIirDlyState );  StdCall; external 'nsp.dll';
   Procedure nspsIirlInitDlyl(var tapStPtr : NSPIirTapState; dlyl : psingle;
                 var dllyStPtr : NSPIirDlyState );  StdCall; external 'nsp.dll';
   Procedure nspcIirlInitDlyl(var tapStPtr : NSPIirTapState; dlyl : pScplx;
                 var dllyStPtr : NSPIirDlyState );  StdCall; external 'nsp.dll';
   Procedure nspdIirlInitDlyl(var tapStPtr : NSPIirTapState; dlyl : pdouble;
                 var dllyStPtr : NSPIirDlyState );  StdCall; external 'nsp.dll';
   Procedure nspzIirlInitDlyl(var tapStPtr : NSPIirTapState; dlyl : pDCplx; 
                 var dllyStPtr : NSPIirDlyState );  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//         Iirl, bIirl               
//                
// Filter a signal through a low-level IIR filter.      
//-------------------------------------------------------------------------
// FUNCTION:      
//   nsp<s,c,d,z>Iirl                
//   nsp<sc,cs,dz,zd>Iirl            
// DESCRIPTION:   
//   filters a single sample throuth a low-level IIR filter and returns    
//   the result.  
// PARAMETERS:    
//   var tapStPtr - input, pointer to the tap state structure;                 
//   var dlyStPtr - input, pointer to the delay line state structure;          
//   smp      - input, the sample value.                
// RETURNS:       
//   the filtering result.           
//                

   Function nspwIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
 samp : SmallInt;    scaleMode : Integer;
                 var scaleFactor) :  SmallInt; StdCall; external 'nsp.dll';
   Function nspsIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : single): Single; StdCall; external 'nsp.dll';
   Function nspcIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : double): double; StdCall; external 'nsp.dll';
   Function nspzIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : DCplx): DCplx; StdCall; external 'nsp.dll';

   Function nspscIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : single): SCplx; StdCall; external 'nsp.dll';
   Function nspcsIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : double): DCplx; StdCall; external 'nsp.dll';
   Function nspzdIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
           samp : DCplx): DCplx; StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
// FUNCTION:
//   nsp<s,c,d,z>bIirl               
//   nsp<sc,cs,dz,zd>bIirl           
// DESCRIPTION:   
//   filters an input block of samples throuth a low-level IIR filter and  
//   returns the result in output block.                
// PARAMETERS:    
//   var tapStPtr - input, the pointer to the tap state structure;             
//   var dlyStPtr - input, the pointer to the delay line state structure;      
//   inSamps  - input, the block of samples;            
//   outSamps - output, the block of filtered samples;  
//   numIters - input, the size of samples' block.      
//                

   Procedure nspwbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
    inSamps : pSmallInt; outSamps : pSmallInt;
          numIters : Integer; scaleMode : Integer;
                var scaleFactor);  StdCall; external 'nsp.dll';
   Procedure nspsbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
    inSamps : pSingle; outSamps : pSingle;
          numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
    inSamps : pSCplx; outSamps : pSCplx;
          numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
    inSamps : pDouble; outSamps : pDouble;
          numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
   inSamps : pDCplx; outSamps : pDCplx;
          numIters : Integer);  StdCall; external 'nsp.dll';

   Procedure nspscbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
     inSamps : pSingle; outSamps : pSCplx;
           numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcsbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
     inSamps : pSCplx; outSamps : pSCplx;
           numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdzbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
     inSamps : pDouble; outSamps : pDCplx;
           numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzdbIirl(var tapStPtr : NSPIirTapState; var dllyStPtr : NSPIirDlyState;
    inSamps : pDCplx; outSamps : pDCplx;
           numIters : Integer);  StdCall; external 'nsp.dll';

   Procedure nspwbLinToALaw(src : pSmallInt;dst : pByte;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspibLinToALaw(src : pInteger; dst : pByte;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbLinToALaw(src : pSingle; dst : pByte;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbLinToALaw(src : pdouble; dst : pByte;len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspwbALawToLin(src : pByte; dst : pSmallInt;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspibALawToLin(src : pByte; dst : pinteger; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbALawToLin(src : pByte; dst : psingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbALawToLin(src : pByte; dst : pdouble; len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspwbLinToMuLaw(src : pSmallInt; dst : pByte;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspibLinToMuLaw(src : pInteger; dst : pByte;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbLinToMuLaw(src : pSingle; dst : pByte;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbLinToMuLaw(src : pdouble; dst : pByte;len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspwbMuLawToLin(src : pByte; dst : pSmallInt;len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspibMuLawToLin(src : pByte; dst : pinteger; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbMuLawToLin(src : pByte; dst : psingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbMuLawToLin(src : pByte; dst : pdouble; len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspMuLawToALaw(src : pByte;
               dst : pByte; len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspALawToMuLaw(src : pByte;
               dst : pByte; len : Integer);  StdCall; external 'nsp.dll';


// --- LMS filter (method) type -------------------------------------------

Type
   NSPLmsType = (NSP_LmsNull, NSP_LmsDefault);

// --- LMS low-level filter state structure -------------------------------

  NSPLmsTapState = Record
    taps : Pointer;
    len : Integer;
    upFactor : Integer;
    upPhase : Integer;
    downFactor : Integer;
    downPhase : Integer;
    isMultiRate : Integer;
    isFilterMode : Integer;
    leak : single;                  // Leak value to range taps magn
    step : single;                  // Step-size to change taps magn
    errDly : integer;                // Delay of error signal
    nspType : NSPLmsType;
  End;

// --- LMS low-level filter delay line ------------------------------------

   NSPLmsDlyState = Record
     dlyl : Pointer;
     len : Integer;
     isFilterMode  : Integer;
     adaptB : Integer;// Is used in delay length definition
   End;

  NSPWLmsTapState = Record
    taps : Pointer;
    len : Integer;
    upFactor : Integer;
    upPhase : Integer;
    downFactor : Integer;
    downPhase : Integer;
    isMultiRate : Integer;
    isFilterMode : Integer;
    leak : SmallInt;// Leak value to range taps magn, not used
    _leakpad : SmallInt;               // for compatible with float leak
    step : SmallInt;// neg. degree of 2 to decrease err*dly
    _steppad : SmallInt;               // for compatible with step : Single
    errDly : Integer;// Delay of error signal
    NSPType : NSPLmsType;
  End;

  NSPWLmsDlyState = NSPLmsDlyState;


  NSPLmsState = Record
   tapState : NSPLmsTapState;
   dlyState : NSPLmsDlyState;
   Case err : integer of
     0: (sVal : single);
     1: (dVal : double);
     2: (cVal : SCplx);
     3: (zVal : DCPLX);
   End;


// ------------------------------------------------------------------------

   Procedure  nspsLmsInit(lmsType : NSPLmsType; tapVals : pSingle;
        tapsLen : Integer; dlyVals : pSingle;
        step : single; leak : single; 
        errDly : Integer; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspdLmsInit(lmsType : NSPLmsType; tapVals : pDouble;
        tapsLen : Integer; dlyVals : pDouble;
        step : single; leak : single; 
        errDly : Integer; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspcLmsInit(lmsType : NSPLmsType; tapVals : pSCplx;
        tapsLen : Integer; dlyVals : pSCplx;
        step : single; leak : single; 
        errDly : Integer; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspzLmsInit(lmsType : NSPLmsType; tapVals : pDCplx;
        tapsLen : Integer; dlyVals : pDCplx;
        step : single; leak : single;
        errDly : Integer; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspscLmsInit(lmsType : NSPLmsType; tapVals : pSCplx;
         tapsLen : Integer;dlyVals : pSingle;
         step : single; leak : single; 
         errDly : Integer; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspdzLmsInit(lmsType : NSPLmsType; tapVals : pDCplx;
         tapsLen : Integer;dlyVals : pDouble;
         step : single; leak : single;
         errDly : Integer; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';


   Procedure  nspsLmsInitMr(lmsType : NSPLmsType; tapVals : pSingle;
          tapsLen : Integer; dlyVals : pSingle; 
          step : single;       leak : single; 
          errDly, downFactor : Integer; 
          downPhase : integer;    var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspdLmsInitMr(lmsType : NSPLmsType; tapVals : pDouble;
          tapsLen : Integer; dlyVals : pDouble;
          step : single;       leak : single;
          errDly, downFactor : Integer; 
          downPhase : integer;    var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspcLmsInitMr(lmsType : NSPLmsType; tapVals : pSCplx;
          tapsLen : Integer; dlyVals : pSCplx;
          step : single;       leak : single;
          errDly, downFactor : Integer; 
          downPhase : integer;    var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspzLmsInitMr(lmsType : NSPLmsType; tapVals : pDCplx; 
          tapsLen : Integer; dlyVals : pDCplx; 
          step : single;       leak : single; 
          errDly, downFactor : Integer; 
          downPhase : integer;    var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspscLmsInitMr(lmsType : NSPLmsType; tapVals : pSCplx;
          tapsLen : Integer; dlyVals : pSingle; 
          step : single;       leak : single; 
          errDly, downFactor : Integer; 
          downPhase : integer;    var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspdzLmsInitMr(lmsType : NSPLmsType; tapVals : pDCplx;
          tapsLen : Integer; dlyVals : pDouble;
          step : single;       leak : single; 
          errDly, downFactor : Integer;
          downPhase : integer;    var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';

   Procedure  nspLmsFree(var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';

   Function nspsLms(var statePtr : NSPLmsState; samp : Single; err : single): Single; StdCall; external 'nsp.dll';
   Function nspdLms(var statePtr : NSPLmsState; samp : double; err : double): double; StdCall; external 'nsp.dll';
   Function nspcLms(var statePtr : NSPLmsState; samp : SCplx; err : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspzLms(var statePtr : NSPLmsState; samp : DCplx; err : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspscLms(var statePtr : NSPLmsState; samp : Single; err : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzLms(var statePtr : NSPLmsState; samp : double; err : DCplx): DCplx; StdCall; external 'nsp.dll';

   Function nspsbLms(var statePtr : NSPLmsState; inSamps : pSingle;err : single): Single; StdCall; external 'nsp.dll';
   Function nspdbLms(var statePtr : NSPLmsState; inSamps : pdouble;err : double): double; StdCall; external 'nsp.dll';
   Function nspcbLms(var statePtr : NSPLmsState; inSamps : pSCplx;err : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspzbLms(var statePtr : NSPLmsState; inSamps : pDCplx;err : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspscbLms(var statePtr : NSPLmsState; inSamps : pSingle;err : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzbLms(var statePtr : NSPLmsState; inSamps : pdouble;err : DCplx): DCplx; StdCall; external 'nsp.dll';

   Procedure  nspsLmsGetTaps(var statePtr : NSPLmsState; outTaps : pSingle);  StdCall; external 'nsp.dll';
   Procedure  nspdLmsGetTaps(var statePtr : NSPLmsState; outTaps : pDouble);  StdCall; external 'nsp.dll';
   Procedure  nspcLmsGetTaps(var statePtr : NSPLmsState; outTaps : pSCplx);  StdCall; external 'nsp.dll';
   Procedure  nspzLmsGetTaps(var statePtr : NSPLmsState; outTaps : pDCplx);  StdCall; external 'nsp.dll';

   Procedure  nspsLmsSetTaps(inTaps : pSingle; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspdLmsSetTaps(inTaps : pDouble; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspcLmsSetTaps(inTaps : pSCplx;  var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspzLmsSetTaps(inTaps : pDCplx; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';

   Procedure  nspsLmsGetDlyl(var statePtr : NSPLmsState; outDlyl : pSingle);  StdCall; external 'nsp.dll';
   Procedure  nspdLmsGetDlyl(var statePtr : NSPLmsState; outDlyl : pDouble);  StdCall; external 'nsp.dll';
   Procedure  nspcLmsGetDlyl(var statePtr : NSPLmsState; outDlyl : pSCplx);  StdCall; external 'nsp.dll';
   Procedure  nspzLmsGetDlyl(var statePtr : NSPLmsState; outDlyl : pDCplx);  StdCall; external 'nsp.dll';

   Procedure  nspsLmsSetDlyl(inDlyl : pSingle; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspdLmsSetDlyl(inDlyl : pDouble;  var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspcLmsSetDlyl(inDlyl : pSCplx;  var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspzLmsSetDlyl(inDlyl : pDCplx;  var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';

   Function nspsLmsGetStep(var statePtr : NSPLmsState): Single; StdCall; external 'nsp.dll';
   Function nspsLmsGetLeak(var statePtr : NSPLmsState): Single; StdCall; external 'nsp.dll';

   Procedure  nspsLmsSetStep(step : single;  var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure  nspsLmsSetLeak(leak : Single;  var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';

   Function nspsLmsDes(var statePtr : NSPLmsstate; samp : Single; des : single): Single; StdCall; external 'nsp.dll';
   Function nspdLmsDes(var statePtr : NSPLmsstate; samp : double; des : double): double; StdCall; external 'nsp.dll';
   Function nspcLmsDes(var statePtr : NSPLmsstate; samp : SCplx; des : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspzLmsDes(var statePtr : NSPLmsstate; samp : DCplx; des : DCplx): DCplx; StdCall; external 'nsp.dll';
   Function nspscLmsDes(var statePtr : NSPLmsstate; samp : Single; des : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzLmsDes(var statePtr : NSPLmsstate; samp : double; des : DCplx): DCplx; StdCall; external 'nsp.dll';

   Procedure nspsbLmsDes(var statePtr : NSPLmsstate; inSamp : pSingle;
       desSamp : pSingle; outSamps : pSingle; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbLmsDes(var statePtr : NSPLmsstate; inSamp : pDouble; 
       desSamp : pDouble; outSamps : pdouble;numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbLmsDes(var statePtr : NSPLmsstate; inSamp : pSCplx;
       desSamp : pSCplx; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbLmsDes(var statePtr : NSPLmsstate; inSamp : pDCplx;
       desSamp : pDCplx;  outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspscbLmsDes(var statePtr : NSPLmsstate; inSamp : pSingle;
       desSamp : pSCplx; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdzbLmsDes(var statePtr : NSPLmsstate; inSamp : pDouble;
       desSamp : pDCplx;  outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';

   Function nspsLmsGetErrVal(var statePtr : NSPLmsState): Single; StdCall; external 'nsp.dll';
   Function nspdLmsGetErrVal(var statePtr : NSPLmsState): double; StdCall; external 'nsp.dll';
   Function nspcLmsGetErrVal(var statePtr : NSPLmsState): SCplx; StdCall; external 'nsp.dll';
   Function nspzLmsGetErrVal(var statePtr : NSPLmsState): DCplx; StdCall; external 'nsp.dll';

   Procedure nspsLmsSetErrVal(err : single; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure nspdLmsSetErrVal(err : double; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure nspcLmsSetErrVal(err : SCplx; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';
   Procedure nspzLmsSetErrVal(err : DCplx; var statePtr : NSPLmsState);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
// LmslInit, LmslInitMr, LmslInitDlyl
//
// Initialize  an adaptive  FIR filter  that uses  the least  mean-square  
// (LMS) algorithm.                  


//--- Single-rate init ----------------------------------------------------

   Procedure nspsLmslInit(lmsType : NSPLmsType;  taps : pSingle;tapsLen : Integer;
step : single;  leak : Single; errDly : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

   Procedure nspdLmslInit(lmsType : NSPLmsType;  taps : pDouble; tapsLen : Integer;
step : single;  leak : Single; errDly : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

   Procedure nspcLmslInit(lmsType : NSPLmsType;  taps : pSCplx; tapsLen : Integer;
step : single;  leak : Single; errDly : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

   Procedure nspzLmslInit(lmsType : NSPLmsType;  taps : pDCplx; tapsLen : Integer;
step : single;  leak : Single; errDly : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';


//--- Multi-rate init -----------------------------------------------------

   Procedure nspsLmslInitMr(lmsType : NSPLmsType;  taps : pSingle;tapsLen : Integer;
         step : single;  leak : Single;  errDly : Integer;  downFactor : Integer;
         downPhase : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

   Procedure nspdLmslInitMr(lmsType : NSPLmsType;  taps : pDouble; tapsLen : Integer;
         step : single;  leak : Single;  errDly : Integer;  downFactor : Integer;
         downPhase : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

   Procedure nspcLmslInitMr(lmsType : NSPLmsType;  taps : pSCplx; tapsLen : Integer;
         step : single;  leak : Single;  errDly : Integer;  downFactor : Integer;
         downPhase : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

   Procedure nspzLmslInitMr(lmsType : NSPLmsType;  taps : pDCplx; tapsLen : Integer;
         step : single;  leak : Single;  errDly : Integer;  downFactor : Integer;
         downPhase : Integer; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

//--- Delay line init -----------------------------------------------------

   Procedure nspsLmslInitDlyl(var tapStPtr : NSPLmsTapstate; dlyl : psingle; adaptB : integer;
           var dlyStPtr : NSPLmsDlyState );  StdCall; external 'nsp.dll';

   Procedure nspdLmslInitDlyl(var tapStPtr : NSPLmsTapstate; dlyl : pdouble; adaptB : integer;
           var dlyStPtr : NSPLmsDlyState );  StdCall; external 'nsp.dll';

   Procedure nspcLmslInitDlyl(var tapStPtr : NSPLmsTapstate; dlyl : pScplx; adaptB : integer;
           var dlyStPtr : NSPLmsDlyState );  StdCall; external 'nsp.dll';

   Procedure nspzLmslInitDlyl(var tapStPtr : NSPLmsTapstate; dlyl : pDCplx;  adaptB : integer;
           var dlyStPtr : NSPLmsDlyState );  StdCall; external 'nsp.dll';

// -------------------------------------------------------------------------
// LmslGetStep, LmslSetStep, LmslGetLeak, LmslSetLeak    
//                 
// Utility  functions  to  get  and  set  the  leak  and step values of a   
// low-level LMS filter.              

   Function nspsLmslGetStep(var statePtr : NSPLmsTapState ): Single; StdCall; external 'nsp.dll';
   Function nspsLmslGetLeak(var statePtr : NSPLmsTapState ): Single; StdCall; external 'nsp.dll';

   Procedure  nspsLmslSetStep(step : single; var statePtr : NSPLmsTapState);  StdCall; external 'nsp.dll';
   Procedure  nspsLmslSetLeak(leak : Single; var statePtr : NSPLmsTapState);  StdCall; external 'nsp.dll';


// -------------------------------------------------------------------------
// Lmsl, bLmsl
//                 
// Filter samples through a low-level LMS filter.        

   Function  nspsLmsl(var tapStPtr : NSPLmsTapstate;
      var dlyStPtr : NSPLmsDlystate; samp : single; err : single): Single; StdCall; external 'nsp.dll';
   Function nspcLmsl(var tapStPtr : NSPLmsTapstate;
      var dlyStPtr : NSPLmsDlystate; samp : SCplx; err : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdLmsl(var tapStPtr : NSPLmsTapstate;
      var dlyStPtr : NSPLmsDlystate; samp : double; err : double): double; StdCall; external 'nsp.dll';
   Function nspzLmsl(var tapStPtr : NSPLmsTapstate;
      var dlyStPtr : NSPLmsDlystate; samp : DCplx; err : DCplx): DCplx; StdCall; external 'nsp.dll';

   Function nspscLmsl(var tapStPtr : NSPLmsTapstate;
      var dlyStPtr : NSPLmsDlystate; samp : Single; err : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzLmsl(var tapStPtr : NSPLmsTapstate;
      var dlyStPtr : NSPLmsDlystate; samp : double; err : DCplx): DCplx; StdCall; external 'nsp.dll';


   Function nspsbLmsl(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pSingle;  err : single): Single; StdCall; external 'nsp.dll';
   Function nspcbLmsl(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pSCplx;  err : SCplx)  : SCplx; StdCall; external 'nsp.dll';
   Function nspdbLmsl(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pdouble;  err : double): double; StdCall; external 'nsp.dll';
   Function nspzbLmsl(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pDCplx;  err : DCplx): DCplx; StdCall; external 'nsp.dll';


   Function nspscbLmsl(var tapStPtr : NSPLmsTapstate;var dlyStPtr : NSPLmsDlystate;
       inSamps : pSingle; err : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzbLmsl(var tapStPtr : NSPLmsTapstate;var dlyStPtr : NSPLmsDlystate;
       inSamps : pdouble; err : DCplx): DCplx; StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
// LmslNa, bLmslNa
//                
// Filter a signal using a LMS filter, but without adapting the filter.    

   Function nspsLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : single): Single; StdCall; external 'nsp.dll';
   Function nspcLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : double): double; StdCall; external 'nsp.dll';
   Function nspzLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : DCplx): DCplx; StdCall; external 'nsp.dll';

   Function nspscLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : single): SCplx; StdCall; external 'nsp.dll';
   Function nspcsLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : SCplx): SCplx; StdCall; external 'nsp.dll';
   Function nspdzLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : double): DCplx; StdCall; external 'nsp.dll';
   Function nspzdLmslNa(var tapStPtr : NSPLmsTapstate;
             var dlyStPtr : NSPLmsDlystate; samp : DCplx): DCplx; StdCall; external 'nsp.dll';


   Procedure nspsbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pSingle; outSamps : pSingle; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pSCplx; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pdouble; outSamps : pdouble; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
      inSamps : pDCplx; outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';


   Procedure nspscbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
       inSamps : pSingle; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcsbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
       inSamps : pSCplx; outSamps : pSCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdzbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
       inSamps : pdouble;outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzdbLmslNa(var tapStPtr : NSPLmsTapstate; var dlyStPtr : NSPLmsDlystate;
       inSamps : pDCplx; outSamps : pDCplx; numIters : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsLmslGetTaps(var tapStPtr : NSPLmsTapstate;outTaps : pSingle);  StdCall; external 'nsp.dll';
   Procedure nspcLmslGetTaps(var tapStPtr : NSPLmsTapstate;outTaps : pSCplx);  StdCall; external 'nsp.dll';
   Procedure nspdLmslGetTaps(var tapStPtr : NSPLmsTapstate;outTaps : pDouble);  StdCall; external 'nsp.dll';
   Procedure nspzLmslGetTaps(var tapStPtr : NSPLmsTapstate;outTaps : pDCplx);  StdCall; external 'nsp.dll';

   Procedure nspsLmslSetTaps(inTaps : pSingle; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';
   Procedure nspcLmslSetTaps(inTaps : pSCplx; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';
   Procedure nspdLmslSetTaps(inTaps : pDouble; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';
   Procedure nspzLmslSetTaps(inTaps : pDCplx; var tapStPtr : NSPLmsTapState);  StdCall; external 'nsp.dll';

   Procedure nspsLmslGetDlyl(var tapStPtr : NSPLmsTapstate;
          var dlyStPtr : NSPLmsDlystate;outDlyl : pSingle);  StdCall; external 'nsp.dll';
   Procedure nspcLmslGetDlyl(var tapStPtr : NSPLmsTapstate;
          var dlyStPtr : NSPLmsDlystate;outDlyl : pSCplx);  StdCall; external 'nsp.dll';
   Procedure nspdLmslGetDlyl(var tapStPtr : NSPLmsTapstate;
          var dlyStPtr : NSPLmsDlystate;outDlyl : pDouble);  StdCall; external 'nsp.dll';
   Procedure nspzLmslGetDlyl(var tapStPtr : NSPLmsTapstate;
          var dlyStPtr : NSPLmsDlystate; outDlyl : pDCplx);  StdCall; external 'nsp.dll';

   Procedure nspsLmslSetDlyl(var tapStPtr : NSPLmsTapstate;inDlyl : pSingle;
                var dlyStPtr : NSPLmsDlyState);  StdCall; external 'nsp.dll';
   Procedure nspcLmslSetDlyl(var tapStPtr : NSPLmsTapstate;inDlyl : pSCplx;
                var dlyStPtr : NSPLmsDlyState);  StdCall; external 'nsp.dll';
   Procedure nspdLmslSetDlyl(var tapStPtr : NSPLmsTapstate;inDlyl : pDouble;
                var dlyStPtr : NSPLmsDlyState);  StdCall; external 'nsp.dll';
   Procedure nspzLmslSetDlyl(var tapStPtr : NSPLmsTapstate;inDlyl : pDCplx;
                var dlyStPtr : NSPLmsDlyState);  StdCall; external 'nsp.dll';

// New flavors for short data type. 
{   Notes:
      1. leak is not used
      2. step is negative degree of 2, so
         w[k] += (err*dly[k+errDly]) >> step;
      3. adaptB is not used, use FIR functions instead.
      4. The structure for short data have "flavored" name now.
      5. About scale mode. I think we must use SATURATE mode only
         for LMS functions, because it is important to save
         sign of the multiply result in a convergence process.
     13. Convergence of LMS adaptation is very difficult job now.
}

   Procedure  nspwLmslInit(lmsType : NSPLmsType;  taps : pSmallInt; tapsLen : Integer;
        step : SmallInt; leak  : SmallInt; errDly : Integer;
        var tapStPtr : NSPWLmsTapState);  StdCall; external 'nsp.dll';

   Procedure  nspwLmslInitDlyl(var tapStPtr : NSPWLmsTapstate; dlyl : pSmallint;
            adaptB : integer; var dlyStPtr : NSPWLmsDlyState);  StdCall; external 'nsp.dll';

   Function nspwLmsl(var tapStPtr : NSPWLmsTapstate; var dlyStPtr : NSPWLmsDlystate;
    samp : SmallInt; err : SmallInt) :  SmallInt; StdCall; external 'nsp.dll';

   Procedure  nspwLmslSetTaps(inTaps : pSmallInt; var tapStPtr : NSPWLmsTapState);  StdCall; external 'nsp.dll';
   Procedure  nspwLmslSetDlyl(var tapStPtr : NSPWLmsTapstate;
           inDlyl : pSmallInt; var dlyStPtr : NSPWLmsDlyState);  StdCall; external 'nsp.dll';

//
//  Contents:
//           nsp?bLn1   -
//   Computes the natural log of each element
//   in the array in-place
//           nsp?bLn2   -
//   Computes the natural log of each element
//   in the array and stores the results
//   in the another array
//           nsp?bExp1  -
//   Computes e to the power of each element
//   in the array in-place
//           nsp?bExp2  -
//   Computes e to the power of each element
//   in the array and stores the results
//   in the another array
//

   Procedure  nspsbLn1(vec : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspsbLn2(src : pSingle; dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspsbExp1(vec : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspsbExp2(src : pSingle;dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspdbLn1(vec : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspdbLn2(src : pdouble; dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspdbExp1(vec : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspdbExp2(src : pdouble; dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbLn1(vec : pSmallInt; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspwbLn2(src : pSmallInt; dst : pSmallInt; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspwbExp1(vec : pSmallInt; len : integer;
     scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';
   Procedure  nspwbExp2(src : pSmallInt; dst : pSmallInt; len : integer;
     scaleMode : Integer; var scaleFactor);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//                
//            Miscellaneous Signal Processing           
//                
//      BitRev, CalcBitRevTbl, GetBitRevTbl, FreeBitRevTbls                
//                
// Obtain bit-reversed numbers, indices, and indexing tables.              
//                

   Function nspBitRev        (a : integer;   order : Integer) : Integer;  StdCall; external 'nsp.dll';
   Function nspGetBitRevTbl  (order : Integer) : pInteger; StdCall; external 'nsp.dll';
   Procedure         nspCalcBitRevTbl (tbl : pInteger;order : Integer);  StdCall; external 'nsp.dll';
   Procedure         nspFreeBitRevTbls; StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//                
//        bBitRev1, bBitRev2         
//                
// Permute a vector into binary bit-reversed order.     
//                

   Procedure  nspwbBitRev1(vec : pSmallInt; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbBitRev1(vec : pWCplx; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspsbBitRev1(vec : pSingle; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcbBitRev1(vec : pSCplx; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspdbBitRev1(vec : pDouble; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbBitRev1(vec : pDCplx; order : Integer);  StdCall; external 'nsp.dll';

   Procedure  nspwbBitRev2(src : pSmallInt; dst : pSmallInt; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspvbBitRev2(src : pWCplx; dst : pWCplx; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspsbBitRev2(src : pSingle;dst : pSingle; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcbBitRev2(src : pSCplx;  dst : pSCplx; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspdbBitRev2(src : pdouble; dst : pDouble; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzbBitRev2(src : pDCplx; dst : pDCplx; order : Integer);  StdCall; external 'nsp.dll';

//-------------------------------------------------------------------------
//                
//        GetFftTwdTbl, GetDftTwdTbl, FreeTwdTbls       
//                
// Compute and store twiddle factors for FFT and DFT computations.         

   Procedure  nspvCalcFftTwdTbl(tbl : pWCplx; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcCalcDftTwdTbl(tbl : pSCplx; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspcCalcFftTwdTbl(tbl : pSCplx; order : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzCalcDftTwdTbl(tbl : pDCplx; len : Integer);  StdCall; external 'nsp.dll';
   Procedure  nspzCalcFftTwdTbl(tbl : pDCplx; order : Integer);  StdCall; external 'nsp.dll';

   Function nspvGetFftTwdTbl(order : Integer) : PWCplx; StdCall; external 'nsp.dll';
   Function nspcGetDftTwdTbl(len : Integer) : pSCplx; StdCall; external 'nsp.dll';
   Function nspcGetFftTwdTbl(order : Integer) : pSCplx; StdCall; external 'nsp.dll';
   Function nspzGetDftTwdTbl(len : Integer) : pDCplx; StdCall; external 'nsp.dll';
   Function nspzGetFftTwdTbl(order : Integer) : pDCplx; StdCall; external 'nsp.dll';

   Procedure  nspvFreeTwdTblstbl; StdCall; external 'nsp.dll';
   Procedure  nspcFreeTwdTblstbl; StdCall; external 'nsp.dll';
   Procedure  nspzFreeTwdTblstbl; StdCall; external 'nsp.dll';

Type
  NSPSRandUniState = Record
   low,muler : Single;             // Lower value and scale multiplier
   seed2,carry : DWORD;           // Variables of the algorithm
   seed1 : Array[0..2] of DWORD;              // Variables of the algorithm
   quickSeed : Integer;
  End;

  NSPDRandUniState = Record
   low,muler : Double;
   seed2,carry : DWORD;           // Variables of the algorithm
   seed1 : Array[0..2] of DWORD;              // Variables of the algorithm
   quickSeed : Integer;
  End;

Const
  XBUFLEN = 32;

{ A new state structure of the pseudo random numbers
   generator has two seed values (m_seed) now. Generator
   based on multiplication uses m_seed values. Generator
   based on subtraction uses d_seed and carry.
}

Type
  NSPWRandUniState = Record
   low,muler : Single;
   center, pwr2 : Integer;
   m_seed : array [0..1] of DWORD;
   d_seed : array [0..2] of DWORD;
   carry : DWORD;
   quickSeed : Integer;
   current : Integer;
   xbuf : Array [0..XBUFLEN-1] of SmallInt;
  End;

  Type
  NSPCRandUniState = NSPSRandUniState;
  NSPZRandUniState = NSPDRandUniState;
  NSPVRandUniState = NSPWRandUniState;


  NSPSRandGausState = Record
   mean,stDev : Single;            // Mean and standard deviation values
   extraValue : Single;            // Value computed early
   useExtra : Integer;              // Flag to use extra value
   seed2,carry : DWORD;           // Variables of the algorithm
   seed1 : Array [0..2] of DWORD;              // Variables of the algorithm
   quickSeed : Integer;
  END;

  NSPDRandGausState = Record
   mean,stDev : Double;
   extraValue : Double;
   useExtra : Integer;
   seed2,carry : DWORD;
   seed1 : array [0..2] of DWORD;
   quickSeed : Integer;
  End;

  NSPWRandGausState = Record
   useExtra : Integer;
   seed2,carry : DWORD;
   seed1 : array [0..2] of DWORD;
   mean,stDev : SmallInt;
   extraValue : SmallInt;
   quickSeed :Integer;
  End;

  NSPCRandGausState = NSPSRandGausState;
  NSPZRandGausState = NSPDRandGausState;
  NSPVRandGausState = NSPWRandGausState;

//-------------------------------------------------------------------------
{
   Uniform distribution.
   Algorithm by G.Marsaglia and A.Zaman (Computers in Physics, vol.8,
   #1, 1994, 117) are used to build generator of uniformly distributed
   random numbers.
}

   Procedure nspsRandUniInit(seed, low, high : Single;
          var statePtr : NSPSRandUniState );  StdCall; external 'nsp.dll';
   Procedure nspcRandUniInit(seed, low, high : Single;
          var statePtr : NSPCRandUniState);  StdCall; external 'nsp.dll';
   Procedure nspdRandUniInit(seed, low, high : Double;
          var statePtr : NSPDRandUniState);  StdCall; external 'nsp.dll';
   Procedure nspzRandUniInit(seed, low, high : Double;
          var statePtr : NSPZRandUniState);  StdCall; external 'nsp.dll';
   Procedure nspwRandUniInit(seed, low, high : DWORD;
          var statePtr : NSPWRandUniState);  StdCall; external 'nsp.dll';
   Procedure nspvRandUniInit(seed, low, high : DWORD;
          var statePtr : NSPVRandUniState);  StdCall; external 'nsp.dll';

   Function nspsRandUni(var statePtr : NSPSRandUniState ): Single; StdCall; external 'nsp.dll';
   Function nspcRandUni(var statePtr : NSPCRandUniState): SCplx; StdCall; external 'nsp.dll';
   Function nspdRandUni(var statePtr : NSPDRandUniState): double; StdCall; external 'nsp.dll';
   Function nspzRandUni(var statePtr : NSPZRandUniState): DCplx; StdCall; external 'nsp.dll';
   Function nspwRandUni(var statePtr : NSPWRandUniState) :  SmallInt; StdCall; external 'nsp.dll';
   Function nspvRandUni(var statePtr : NSPVRandUniState): WCplx; StdCall; external 'nsp.dll';

   Procedure nspsbRandUni(var statePtr : NSPSRandUniState;
       samps : pSingle; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbRandUni(var statePtr : NSPCRandUniState;
       samps : pSCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbRandUni(var statePtr : NSPDRandUniState;
       samps : pdouble; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbRandUni(var statePtr : NSPZRandUniState;
       samps : pDCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbRandUni(var statePtr : NSPWRandUniState;
       samps : pSmallInt; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbRandUni(var statePtr : NSPVRandUniState;
       samps : pWCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';

//
{   Normal distribution.
   Algorithm by G.Box and M.Muller and by G.Marsaglia (Reference:
   D.Knuth. The Art of Computer Programming. vol.2, 1969) are used
   to build generator of normally distributed random numbers.
}


   Procedure nspsRandGausInit(seed, mean, stDev : Single;
           var statePtr : NSPSRandGausState );  StdCall; external 'nsp.dll';
   Procedure nspcRandGausInit(seed, mean, stDev : Single;
           var statePtr : NSPCRandGausState);  StdCall; external 'nsp.dll';
   Procedure nspdRandGausInit(seed, mean, stDev : Double;
           var statePtr : NSPDRandGausState);  StdCall; external 'nsp.dll';
   Procedure nspzRandGausInit(seed, mean, stDev : Double;
           var statePtr : NSPDRandGausState);  StdCall; external 'nsp.dll';
   Procedure nspwRandGausInit(seed, mean, stDev : Smallint;
           var statePtr : NSPWRandGausState);  StdCall; external 'nsp.dll';
   Procedure nspvRandGausInit(seed, mean, stDev : Smallint;
           var statePtr : NSPVRandGausState);  StdCall; external 'nsp.dll';

   Function nspsRandGaus(var statePtr : NSPSRandGausState): Single; StdCall; external 'nsp.dll';
   Function nspcRandGaus(var statePtr : NSPCRandGausState): SCplx; StdCall; external 'nsp.dll';
   Function nspdRandGaus(var statePtr : NSPDRandGausState): double; StdCall; external 'nsp.dll';
   Function nspzRandGaus(var statePtr : NSPDRandGausState): DCplx; StdCall; external 'nsp.dll';
   Function nspwRandGaus(var statePtr : NSPWRandGausState) :  SmallInt; StdCall; external 'nsp.dll';
   Function nspvRandGaus(var statePtr : NSPVRandGausState): WCplx; StdCall; external 'nsp.dll';

   Procedure nspsbRandGaus(var statePtr : NSPSRandGausstate;
        samps : pSingle;  sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbRandGaus(var statePtr : NSPCRandGausstate;
        samps : pSCplx;  sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbRandGaus(var statePtr : NSPDRandGausstate;
        samps : pdouble; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbRandGaus(var statePtr : NSPDRandGausstate;
        samps : pDCplx;  sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbRandGaus(var statePtr : NSPWRandGausstate;
        samps : pSmallInt;  sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbRandGaus(var statePtr : NSPVRandGausstate;
        samps : pWCplx;  sampsLen : Integer);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        UpSample
//
// Up-sample  a  signal, conceptually increasing  its  sampling rate by an
// integer factor.

    Procedure nspsUpSample(factor : Integer; phase : Integer; src : pSingle;srcLen : Integer;
        var dst : Single);  StdCall; external 'nsp.dll';
    Procedure nspcUpSample(factor : Integer; phase : Integer; src : pSCplx; srcLen : Integer;
        dest : pSCplx);  StdCall; external 'nsp.dll';
    Procedure nspdUpSample(factor : Integer; phase : Integer; src : pdouble;srcLen : Integer;
        dest : pDouble);  StdCall; external 'nsp.dll';
    Procedure nspzUpSample(factor : Integer; phase : Integer; src : pDCplx;srcLen : Integer;
        dst : pDCplx);  StdCall; external 'nsp.dll';
    Procedure nspwUpSample(factor : Integer; phase : Integer; src : pSmallInt;srcLen : Integer;
        dst : pSmallInt);  StdCall; external 'nsp.dll';
    Procedure nspvUpSample(factor : Integer; phase : Integer; src : pWCplx;srcLen : Integer;
        dst : pWCplx);  StdCall; external 'nsp.dll';

// ------------------------------------------------------------------------
//        DownSample                 
//                
// Down-sample a  signal, conceptually decreasing  its sample rate  by an  
// integer factor.

   Procedure nspsDownSample(factor : Integer; phase : Integer; src : pSingle;srcLen : Integer;
         var dst : Single);  StdCall; external 'nsp.dll';
   Procedure nspcDownSample(factor : Integer; phase : Integer; src : pSCplx; srcLen : Integer;
         dest : pSCplx);  StdCall; external 'nsp.dll';
   Procedure nspdDownSample(factor : Integer; phase : Integer; src : pdouble;srcLen : Integer;
         dest : pDouble);  StdCall; external 'nsp.dll';
   Procedure nspzDownSample(factor : Integer; phase : Integer; src : pDCplx;srcLen : Integer;
         dst : pDCplx);  StdCall; external 'nsp.dll';
   Procedure nspwDownSample(factor : Integer; phase : Integer; src : pSmallInt;srcLen : Integer;
         dst : pSmallInt);  StdCall; external 'nsp.dll';
   Procedure nspvDownSample(factor : Integer; phase : Integer; src : pWCplx;srcLen : Integer;
         dst : pWCplx);  StdCall; external 'nsp.dll';

Type

  NSPSToneState = Record
   cosBase : Double;
   cosCurr : Double;
   cosPrev : double;
   mag : Single;
   rfreq : Single;
   phase : Single;
   isInit : Array [0..3] of Char;
  End;

  NSPCToneState = Record
   cosBase : Double;
   cosCurr : DCplx;
   cosPrev : DCplx;
   mag : Single;
   rfreq : Single;
   phase : Single;
   isInit : Array [0..3] of Char;
  End;

  NSPDToneState = Record
   cosBase : Double;
   cosCurr : Double;
   cosPrev : double;
   mag : Double;
   rfreq : Double;
   phase : Double;
   isInit : Array [0..3] of Char;
 End;

  NSPZToneState = Record
   cosBase : Double;
   cosCurr : DCplx;
   cosPrev : DCplx;
   mag : Double;
   rfreq : Double;
   phase : Double;
   isInit : Array [0..3] of Char;
  End;

  NSPWToneState = Record
   dummy : Array [0..31] of double;
  End;

  NSPVToneState = Record
   dummy : Array [0..31] of double;
  End;

//--- Initialization ------------------------------------------------------

   Procedure nspwToneInit(rfreq, phase : single; mag : smallint;
       var statePtr :NSPWToneState);  StdCall; external 'nsp.dll';

   Procedure nspvToneInit(rfreq, phase : single; mag : smallint;
       var statePtr : NSPVToneState);  StdCall; external 'nsp.dll';

   Procedure nspsToneInit(rfreq, phase, mag : single;
       var statePtr : NSPSToneState);  StdCall; external 'nsp.dll';

   Procedure nspcToneInit(rfreq, phase, mag : single;
       var statePtr : NSPCToneState);  StdCall; external 'nsp.dll';

   Procedure nspdToneInit(rfreq : Double; phase : double; mag : Double;
       var statePtr : NSPDToneState);  StdCall; external 'nsp.dll';

   Procedure nspzToneInit(rfreq : Double; phase : double; mag : Double;
       var statePtr : NSPZToneState);  StdCall; external 'nsp.dll';

//--- Dot product tone functions ------------------------------------------

   Function nspwTone(var statePtr : NSPWToneState) :  SmallInt; StdCall; external 'nsp.dll';
   Function nspvTone(var statePtr : NSPVToneState): WCplx; StdCall; external 'nsp.dll';
   Function nspsTone(var statePtr : NSPSToneState): Single; StdCall; external 'nsp.dll';
   Function nspcTone(var statePtr : NSPCToneState): SCplx; StdCall; external 'nsp.dll';
   Function nspdTone(var statePtr : NSPDToneState): double; StdCall; external 'nsp.dll';
   Function nspzTone(var statePtr : NSPZToneState): DCplx; StdCall; external 'nsp.dll';

//--- Array product tone functions ----------------------------------------

   Procedure nspwbTone(var statePtr : NSPWToneState; samps : pSmallInt; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbTone(var statePtr : NSPVToneState; samps : pWCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbTone(var statePtr : NSPSToneState; samps : pSingle; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbTone(var statePtr : NSPCToneState; samps : pSCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbTone(var statePtr : NSPDToneState; samps : pdouble; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbTone(var statePtr : NSPZToneState; samps : pDCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';

//
//  Purpose: Generating of signals with triangle wave form.
//           Provides samples of a triangle of arbitrary frequency,
//           phase, magnitude, and asymmetry.
//
//           0 =< phase < 2PI,  -PI < asym < PI,
//           mag > 0, 0 =< rfrec < 0.5
//
//           asym = 0       => Triangle is symmetric (old version)
//           asym = -PI+eps => Triangle with sheer back (0<eps)
//           asym =  PI-eps => Triangle with sheer front-fore
//
//           Like (cos) tone functions the triangle has period equal 2PI
//           Triangle phase is given in radians.
//
//--- Triangle structures -------------------------------------------------

Type
  NSPWTrnglState  = Record
   mag : Single;
   step : Single;
   step1 : Single;
   step2 : Single;
   st12 : single;
   st21 : single;
   last : Single;
 End;

  NSPVTrnglState  = Record
   mag : Single;
   step : SCplx;
   step1 : Single;
   step2 : Single;
   st12 : single;
   st21 : single;
   last : SCplx;
 End;

  NSPITrnglState  = Record
   mag : Double;
   step : double;
   step1 : double;
   step2 : double;
   st12 : Double;
   st21 : Double;
   last : Double;
 End;

  NSPJTrnglState  = Record
   mag : Double;
   step : DCplx;
   step1 : double;
   step2 : double;
   st12 : Double;
   st21 : Double;
   last : DCplx;
 End;

  NSPSTrnglState  = Record
   mag : Single;
   step : Single;
   step1 : Single;
   step2 : Single;
   st12 : single;
   st21 : single;
   last : Single;
 End;

  NSPCTrnglState  = Record
   mag : Single;
   step : SCplx;
   step1 : Single;
   step2 : Single;
   st12 : single;
   st21 : single;
   last : SCplx;
 End;

  NSPDTrnglState  = Record
   mag : Double;
   step : double;
   step1 : double;
   step2 : double;
   st12 : Double;
   st21 : Double;
   last : Double;
 End;

  NSPZTrnglState  = Record
   mag : Double;
   step : DCplx;
   step1 : double;
   step2 : double;
   st12 : Double;
   st21 : Double;
   last : DCplx;
 End;

 //--- Triangle initialization ---------------------------------------------

   Procedure nspwTrnglInit(rfreq, phase : single; mag : smallint;
        asym : Single;  var stptr : NSPWTrnglState);  StdCall; external 'nsp.dll';
   Procedure nspvTrnglInit(rfreq, phase : single; mag : smallint;
        asym : Single;  var stptr : NSPVTrnglState);  StdCall; external 'nsp.dll';
   Procedure nspsTrnglInit(rfrq, phase, mag : single;
        asym : Single;  var stptr : NSPSTrnglState);  StdCall; external 'nsp.dll';
   Procedure nspdTrnglInit(rfrq, phase, mag : double;
        asym : Double; var stptr : NSPDTrnglState);  StdCall; external 'nsp.dll';
   Procedure nspcTrnglInit(rfrq, phase, mag : single;
        asym : Single;  var stptr : NSPCTrnglState);  StdCall; external 'nsp.dll';
   Procedure nspzTrnglInit(rfrq, phase, mag : double;
        asym : Double; var stptr : NSPZTrnglState);  StdCall; external 'nsp.dll';

//--- Single-Sample-Generating triangle functions -------------------------

   Function nspwTrngl(var statePtr : NSPWTrnglState) :  SmallInt; StdCall; external 'nsp.dll';
   Function nspvTrngl(var statePtr : NSPVTrnglState): WCplx; StdCall; external 'nsp.dll';
   Function nspsTrngl(var statePtr : NSPSTrnglState): Single; StdCall; external 'nsp.dll';
   Function nspdTrngl(var statePtr : NSPDTrnglState): double; StdCall; external 'nsp.dll';
   Function nspcTrngl(var statePtr : NSPCTrnglState): SCplx; StdCall; external 'nsp.dll';
   Function nspzTrngl(var statePtr : NSPZTrnglState): DCplx; StdCall; external 'nsp.dll';

//--- Block-of-Samples-Generating triangle functions ----------------------

   Procedure nspwbTrngl(var statePtr : NSPWTrnglState; samps : pSmallInt; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbTrngl(var statePtr : NSPVTrnglState; samps : pWCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspsbTrngl(var statePtr : NSPSTrnglState; samps : pSingle; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbTrngl(var statePtr : NSPDTrnglState; samps : pdouble; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbTrngl(var statePtr : NSPCTrnglState; samps : pSCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbTrngl(var statePtr : NSPZTrnglState; samps : pDCplx; sampsLen : Integer);  StdCall; external 'nsp.dll';


Const
    NSP_Gt = 1;
    NSP_Lt = 0;
    HUGE_VAL_S = 3.402823466e+38;
    HUGE_VAL_D = 1.7976931348623158e+308;

// ------------------------------------------------------------------------

   Procedure nspsbSqr1(     vec : pSingle;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSqr1(     vec : pSCplx;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbSqr1(     vec : pDouble;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbSqr1(     vec : pDCplx;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbSqr1(     vec : pSmallInt;  len : integer;
           scaleMode : Integer;        var scaleFactor : Integer );  StdCall; external 'nsp.dll';

   Procedure nspvbSqr1(      vec : pWCplx;  len : integer;
           scaleMode : Integer;        var scaleFactor : Integer );  StdCall; external 'nsp.dll';

   Procedure nspsbSqr2(src : pSingle;dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSqr2(src : pSCplx;  dst : pSCplx; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbSqr2(src : pdouble; dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbSqr2(src : pDCplx; dst : pDCplx; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbSqr2(src : pSmallInt; dst : pSmallInt; len : integer;
           scaleMode : Integer;        var scaleFactor);  StdCall; external 'nsp.dll';
   Procedure nspvbSqr2(src : pWCplx; dst : pWCplx; len : integer;
           scaleMode : Integer;        var scaleFactor : Integer );  StdCall; external 'nsp.dll';

   Procedure nspsbSqrt1(     vec : pSingle;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSqrt1(     vec : pSCplx;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbSqrt1(     vec : pDouble;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbSqrt1(     vec : pDCplx;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbSqrt1(     vec : pSmallInt;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbSqrt1(      vec : pWCplx;  len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsbSqrt2(src : pSingle;dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbSqrt2(src : pSCplx;  dst : pSCplx; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbSqrt2(src : pdouble; dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbSqrt2(src : pDCplx; dst : pDCplx; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbSqrt2(src : pSmallInt; dst : pSmallInt; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbSqrt2(src : pWCplx; dst : pWCplx; len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsbThresh1(vec : pSingle; len : integer; thresh : Single; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbThresh1(vec : pSCplx; len : integer; thresh : Single; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbThresh1(vec : pDouble; len : integer; thresh : Double; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbThresh1(vec : pDCplx; len : integer; thresh : Double; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbThresh1(vec : pSmallInt; len : integer; thresh : SmallInt; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbThresh1(vec : pWCplx; len : integer; thresh : SmallInt; relOp :Integer);  StdCall; external 'nsp.dll';

   Procedure nspsbThresh2(src : pSingle;   dst : pSingle;len : integer;
             thresh : Single; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspcbThresh2(src : pSCplx;     dst : pSCplx;len : integer;
             thresh : Single; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbThresh2(src : pdouble;    dst : pDouble;len : integer;
             thresh : Double; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspzbThresh2(src : pDCplx;    dst : pDCplx;len : integer;
             thresh : Double; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbThresh2(src : pSmallInt;    dst : pSmallInt;len : integer;
             thresh : SmallInt; relOp :Integer);  StdCall; external 'nsp.dll';
   Procedure nspvbThresh2(src : pWCplx;    dst : pWCplx;len : integer;
             thresh : SmallInt; relOp :Integer);  StdCall; external 'nsp.dll';

   Procedure nspsbInvThresh1(vec : pSingle; len : integer; thresh : Single);  StdCall; external 'nsp.dll';
   Procedure nspcbInvThresh1(vec : pSCplx; len : integer; thresh : Single);  StdCall; external 'nsp.dll';
   Procedure nspdbInvThresh1(vec : pDouble; len : integer; thresh : Double);  StdCall; external 'nsp.dll';
   Procedure nspzbInvThresh1(vec : pDCplx; len : integer; thresh : Double);  StdCall; external 'nsp.dll';

   Procedure nspsbInvThresh2(src : pSingle;dst : pSingle; len : integer;
                thresh : Single);  StdCall; external 'nsp.dll';
   Procedure nspcbInvThresh2(src : pSCplx; dst : pSCplx; len : integer;
                thresh : Single);  StdCall; external 'nsp.dll';
   Procedure nspdbInvThresh2(src : pdouble; dst : pDouble; len : integer;
                thresh : Double);  StdCall; external 'nsp.dll';
   Procedure nspzbInvThresh2(src : pDCplx; dst : pDCplx; len : integer;
                thresh : Double);  StdCall; external 'nsp.dll';

   Procedure nspsbAbs1(vec : pSingle;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbAbs1(vec : pDouble;  len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbAbs1(vec : pSmallInt;  len : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsbAbs2(src : pSingle;dst : pSingle; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdbAbs2(src : pdouble; dst : pDouble; len : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwbAbs2(src : pSmallInt; dst : pSmallInt; len : Integer);  StdCall; external 'nsp.dll';

   Function nspsMax(vec : pSingle;  len : Integer): Single; StdCall; external 'nsp.dll';
   Function nspdMax(vec : pDouble;  len : Integer): double; StdCall; external 'nsp.dll';
   Function nspwMax(vec : pSmallint;  len : Integer) :  SmallInt; StdCall; external 'nsp.dll';

   Function nspsMean(vec : pSingle;  len : Integer): Single; StdCall; external 'nsp.dll';
   Function nspdMean(vec : pDouble;  len : Integer): double; StdCall; external 'nsp.dll';
   Function nspwMean(vec : pSmallint;  len : Integer) :  SmallInt; StdCall; external 'nsp.dll';

   Function nspsMin(vec : pSingle;  len : Integer): Single; StdCall; external 'nsp.dll';
   Function nspdMin(vec : pDouble;  len : Integer): double; StdCall; external 'nsp.dll';
   Function nspwMin(vec : pSmallint;  len : Integer) :  SmallInt; StdCall; external 'nsp.dll';

   Function nspsStdDev(vec : pSingle;  len : Integer): Single; StdCall; external 'nsp.dll';
   Function nspdStdDev(vec : pDouble;  len : Integer): double; StdCall; external 'nsp.dll';
   Function nspwStdDev(vec : pSmallint;  len : integer;
              scaleMode : Integer;        var scaleFactor): Smallint;  StdCall; external 'nsp.dll';
// ------------------------------------------------------------------------
//
//   Win
//
// Multiply a vector by a windowing function.
//                

   Procedure nspsWinBartlett(vec : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcWinBartlett(vec : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdWinBartlett(vec : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzWinBartlett(vec : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwWinBartlett(vec : pSmallInt; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvWinBartlett(vec : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsWinHann(vec : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcWinHann(vec : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdWinHann(vec : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzWinHann(vec : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwWinHann(vec : pSmallInt; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvWinHann(vec : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsWinHamming(vec : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcWinHamming(vec : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdWinHamming(vec : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzWinHamming(vec : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwWinHamming(vec : pSmallInt; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvWinHamming(vec : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsWinBlackman(vec : pSingle; n : Integer; alpha : single);  StdCall; external 'nsp.dll';
   Procedure nspcWinBlackman(vec : pSCplx; n : Integer; alpha : single);  StdCall; external 'nsp.dll';
   Procedure nspdWinBlackman(vec : pDouble; n : Integer; alpha : double);  StdCall; external 'nsp.dll';
   Procedure nspzWinBlackman(vec : pDCplx; n : Integer; alpha : double);  StdCall; external 'nsp.dll';
   Procedure nspwWinBlackman(vec : pSmallInt; n : Integer; alpha : single);  StdCall; external 'nsp.dll';
   Procedure nspvWinBlackman(vec : pWCplx; n : Integer; alpha : single);  StdCall; external 'nsp.dll';

   Procedure nspsWinBlackmanStd(vec : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcWinBlackmanStd(vec : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdWinBlackmanStd(vec : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzWinBlackmanStd(vec : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwWinBlackmanStd(vec : pSmallInt; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvWinBlackmanStd(vec : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsWinBlackmanOpt(vec : pSingle; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspcWinBlackmanOpt(vec : pSCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspdWinBlackmanOpt(vec : pDouble; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspzWinBlackmanOpt(vec : pDCplx; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspwWinBlackmanOpt(vec : pSmallInt; n : Integer);  StdCall; external 'nsp.dll';
   Procedure nspvWinBlackmanOpt(vec : pWCplx; n : Integer);  StdCall; external 'nsp.dll';

   Procedure nspsWinKaiser(vec : pSingle; n : Integer; beta : Single);  StdCall; external 'nsp.dll';
   Procedure nspcWinKaiser(vec : pSCplx; n : Integer; beta : Single);  StdCall; external 'nsp.dll';
   Procedure nspdWinKaiser(vec : pDouble; n : Integer; beta : Double);  StdCall; external 'nsp.dll';
   Procedure nspzWinKaiser(vec : pDCplx; n : Integer; beta : Double);  StdCall; external 'nsp.dll';
   Procedure nspwWinKaiser(vec : pSmallInt; n : Integer; beta : Single);  StdCall; external 'nsp.dll';
   Procedure nspvWinKaiser(vec : pWCplx; n : Integer; beta : Single);  StdCall; external 'nsp.dll';

   Function NSPsDegToRad(deg : single) : single;
   Function NSPdDegToRad(deg : double) : Double;

implementation

  Function NSPsDegToRad(deg : single) : single;
Begin
  Result := (deg/180.0 *NSP_PI);
End;

  Function NSPdDegToRad(deg : double) : double;
Begin
  Result := (deg/180.0 *NSP_PI);
End;

end.
