unit UDAMonitor;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, Buttons,
  ExtCtrls, HRTimer, IntelNSP4,
  ACMIn, ACMConvertor, ACMOut, MMSystem, DropSource, DropTarget,
  UPlayerComponent, jpeg;

const
 nDFTMax  = 2048;
 NBINS = 13;           // The number of spectral displays

 maxChannels = 2;
 noiseNormalFactor = 60;
 noisePeakFactor = 5;
 noisePeakDuration = 5;
 noiseMaxNormalDuration = 20;
 noiseMinNormalDuration = 2;

 filterCodeLP = 1;
 filterCodeHP = 2;
 filterCodeNoise = 4;

 maxTrackBar = 1000;

 standardHeight = 235;
 imageHeight = 240;
 trackBarLeft=105;
 trackBarTop=80;
 trackBarWidth=230;

type
 FilterConstants8Pole = record
   filterGain: real;
   yFactor: array [0..7] of real;
 end;


 FilterConstants8PoleArray = array [0..5] of  FilterConstants8Pole;

 FilterDataStruct8Pole = record
   xv, yv: array [0..8] of real;
 end;


 FilterConstants2Pole = record
   filterGain: real;
   yFactor: array [0..1] of real;
 end;


 FilterConstants2PoleArray = array [0..5] of  FilterConstants2Pole;

 FilterDataStruct2Pole = record
   xv, yv: array [0..2] of real;
 end;

 TNoiseMode = (NoNoise, NormalNoise, IncreaseNoise, DecreaseNoise );

const
{ The following values are taken from:
   Digital filters designed by mkfilter/mkshape/gencode   A.J. Fisher
   Command line: /www/usr/fisher/helpers/mkfilter -Bu -Lp -o 8 -a 7.2916666667e-02 0.0000000000e+00 -l
   Website: http://www-users.cs.york.ac.uk/~fisher/mkfilter/trad.html
}
 filterLP_3500_8PoleAtRate: FilterConstants8PoleArray =
(
 ( filterGain: 3.753910148e+05;
   yFactor: (-0.0929715428, 0.9619117484, -4.3939983044, 11.5844006330,
             -19.2987699490, 20.8284307370, -14.2433823140, 5.6536970354) ),
 ( filterGain: 2.067052154e+05;
   yFactor: (-0.0749391665, 0.7908632106, -3.6902369723, 9.9541566835,
             -16.9987027290, 18.8477879950, -13.2767646700, 5.4465971703) ),
 ( filterGain: 2.269725954e+04;
   yFactor: (-0.0268965572, 0.3097392114, -1.5888819744, 4.7528602453,
             -9.0940029435, 11.4399227870, -9.2880746605, 4.4840549977) ),
 ( filterGain: 3.352882664e+03;
   yFactor: (-0.0072705699, 0.0922045375, -0.5263006177, 1.7733545328,
             -3.8815833632, 5.6927329501, -5.5372122973, 3.3177226182) ),
 ( filterGain: 1.903920528e+03;
   yFactor: (-0.0044169397, 0.0578777317, -0.3428030924, 1.2038068327,
             -2.7648902233, 4.2832884751, -4.4617416046, 2.8944194268) ),
 ( filterGain: 2.498889308e+02;
   yFactor: (-0.0003942256, 0.0054033468, -0.0413613426, 0.1535095257,
             -0.4998888290, 0.8242927636, -1.4598833598, 0.9938669792) )
);


{ The following values are taken from:

   Digital filter designed by mkfilter/mkshape/gencode   A.J. Fisher
   Command line: /www/usr/fisher/helpers/mkfilter -Ch -2.0000000000e-01 -Hp -o 2 -a 1.0416666667e-02 0.0000000000e+00 -l
   Chebyshev Highpass -1 dB ripple, 2 pole
}
 filterHP_500_2PoleAtRate: FilterConstants2PoleArray =
(
 ( filterGain: 1.033566839e+00;
   yFactor: (-0.9369275475, 1.9331656568) ),
 ( filterGain: 1.036631392e+00;
   yFactor: (-0.9315481445, 1.9271040531) ),
 ( filterGain: 1.051103083e+00;
   yFactor: (-0.9069281854, 1.8985977089) ),
 ( filterGain: 1.069156046e+00;
   yFactor: (-0.8779232774, 1.8633453765) ),
 ( filterGain: 1.075668166e+00;
   yFactor: (-0.8678968809, 1.8507220435) ),
 ( filterGain: 1.106863354e+00;
   yFactor: (-0.8228060492, 1.7910095016) )
);

type
  TDATrackBar = class(TTrackbar)
                 published
                  property OnMouseDown;
                  property OnMouseUp;
                end;

  TDAMonitorForm = class(TForm)
    ProgressBar1: TProgressBar;
    ProgressBar2: TProgressBar;
    ProgressBar3: TProgressBar;
    ProgressBar4: TProgressBar;
    ProgressBar5: TProgressBar;
    ProgressBar6: TProgressBar;
    ProgressBar7: TProgressBar;
    ProgressBar8: TProgressBar;
    LiveTimer: TTimer;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Bevel1: TBevel;
    ACMO: TACMOut;
    ACMI: TACMIn;
    Bevel2: TBevel;
    ProgressBar9: TProgressBar;
    ProgressBar10: TProgressBar;
    ProgressBar11: TProgressBar;
    ProgressBar12: TProgressBar;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label12: TLabel;
    CheckBoxAMRadio: TCheckBox;
    CheckBoxNoise: TCheckBox;
    OpenDialog1: TOpenDialog;
    SpeedButtonLiveInput: TSpeedButton;
    BtnPlayFile: TButton;
    DAPlayComponent1: TDAPlayComponent;
    DropFileTarget1: TDropFileTarget;
    RadioImage: TImage;
    VUMeterBar: TProgressBar;
    FilenameLabel: TLabel;
    BtnStop: TButton;
    procedure LiveTimerTimer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ACMIBufferFull(Sender: TObject; Data: Pointer;
      Size: Integer);
    procedure CheckBoxNoiseClick(Sender: TObject);
    procedure CheckBoxAMRadioClick(Sender: TObject);
    procedure SpeedButtonLiveInputClick(Sender: TObject);
    procedure DropFileTarget1Drop(Sender: TObject; ShiftState: TShiftState;
      Point: TPoint; var Effect: Integer);
    procedure TrackBar1MouseUp(Sender: TObject; Button: TMouseButton;
                               Shift: TShiftState; X, Y: Integer);
    procedure TrackBar1MouseDown(Sender: TObject; Button: TMouseButton;
                                 Shift: TShiftState; X, Y: Integer);
    procedure JumpToPos();
    procedure DAPlayComponent1RemainingTime(Sender: TObject;
      val: Cardinal);
    procedure DAPlayComponent1TerminateItem(Sender: TObject;
      val: Cardinal);
    procedure DAPlayComponent1FileposTime(Sender: TObject; val: Cardinal);
    procedure DAPlayComponent1EndPlayback(Sender: TObject; val: Cardinal);
    procedure DAPlayComponent1JumpOk(Sender: TObject);
    procedure DAPlayComponent1VULevelChange(Sender: TObject; val: Integer);
    procedure BtnStopClick(Sender: TObject);
    procedure BtnPlayFileClick(Sender: TObject);
  private
    procedure SpectrumVisible(bVal: Boolean);
    procedure PlaybackControlVisible(bVal: Boolean);
    procedure SetLiveInput(bVal: Boolean);
    function GetNoise(ch: Cardinal) : Integer;
    function GetFilteredValue(ch: Cardinal; value: Integer) : Integer;
    procedure FilteredCopy(Source,Dest: pSmallInt; Count: Integer);
    procedure PlayFile(const fName: String);
  private
    TrackBar1: TDATrackBar;
    pDataBuf: Pointer;
    dataSize: Integer;
    fData8: array [0..maxChannels-1] of FilterDataStruct8Pole;
    fData2: array [0..maxChannels-1] of FilterDataStruct2Pole;
    filterCode: Cardinal;
    eNoiseMode: TNoiseMode;
    noiseStateCnt: Cardinal;
    noiseFactor: Cardinal;
    channels: Cardinal;
    samplesPerSec: Cardinal;
    saveNoiseEnabled: Boolean;

    stopped: Boolean;
    bRepositioning: Boolean;
    trackBarPos,desiredTrackbarPos: Integer;
    curFileLen,filePos: Cardinal;
    bDragPos: Boolean;

    HRT : THRTimer;
    Samples : Array[0..nDFTMax+2] of SmallInt;
    Spectrum : Array[0..nDFTMax+2] of Double;
    TimeDFT : Double;
    ab : Array [0..NBINS] of Integer;
    nDFTSize  : Integer;      // Must be a power of 2!!!
  public
    { Public declarations }
     procedure CreateDFT;
     procedure DisplaySpectrum;

  end;


var
  DAMonitorForm: TDAMonitorForm;


//-------------------------------------------------------------------
implementation

{$R *.DFM}

procedure TDAMonitorForm.FormCreate(Sender: TObject);
var
 i,chInx: Integer;
 Period : Extended;
begin
   //  For timing the DFT and spectrum displays.
  HRT := THRTimer.Create;

  nDFTSize := 512;
  ACMI.BufferSize := 2048;
  dataSize := ACMI.BufferSize;
  GetMem(pDataBuf, ACMI.BufferSize);

  saveNoiseEnabled := False;
  filterCode:= 0; // Set no filters by default

  for chInx := 0 to maxChannels -1 do
   begin
     for i:=0 to 7 do
       fData8[chInx].xv[i] := 0.0;
     for i:=0 to 7 do
       fData8[chInx].yv[i] := 0.0;
     for i:=0 to 1 do
       fData2[chInx].xv[i] := 0.0;
     for i:=0 to 1 do
       fData2[chInx].yv[i] := 0.0;
   end;

  channels := 1;
  samplesPerSec := 44100;

  eNoiseMode := NoNoise;
  noiseStateCnt := 0;
  noiseFactor := noiseNormalFactor;

  Period := samplesPerSec/nDFTSize;
  ab[0] := 0;
  ab[1] := Round(100/Period);
  if (ab[1] = 0) then ab[1] := 1;
  For i := 0 to NBINS-1 do
    ab[i+1] := Round(ab[1]*exp(i*ln(nDFTSize/2/ab[1])/(NBINS-1)));

  Label1.Caption := Format('%1.0n Hz',[ab[1]*Period]);
  Label2.Caption := Format('%1.0n Hz',[ab[2]*Period]);
  Label3.Caption := Format('%1.0n Hz',[ab[3]*Period]);
  Label4.Caption := Format('%1.0n Hz',[ab[4]*Period]);
  Label5.Caption := Format('%1.0n Hz',[ab[5]*Period]);
  Label6.Caption := Format('%1.0n Hz',[ab[6]*Period]);
  // The remaining values are in kHz
  Label7.Caption := Format('%1.0n',[ab[7]*Period]);
  Label8.Caption := Format('%5.0n',[ab[8]*Period]);
  Label9.Caption := Format('%1.0n',[ab[9]*Period]);
  Label10.Caption := Format('%1.0n',[ab[10]*Period]);
  Label11.Caption := Format('%1.0n',[ab[11]*Period]);
  Label12.Caption := Format('%5.0n',[ab[12]*Period]);

  stopped:=True;
  bDragPos:=False;
  TrackBar1 := TDATrackBar.Create(Self);
  with TrackBar1 do
   begin
    Parent:=Self;
    Left:=trackBarLeft;
    Top:=trackBarTop;
    OnMouseUp:=TrackBar1MouseUp;
    OnMouseDown:=TrackBar1MouseDown;
    Width:=trackBarWidth;
    Height:=25;
    LineSize:=50;
    Max:=maxTrackBar;
    Orientation:=trHorizontal;
    PageSize:=50;
    Frequency:=50;
    Position:=0;
    SelEnd:=0;
    SelStart:=0;
    TabOrder:=0;
    TickMarks:=tmBottomRight;
    TickStyle:=tsAuto;
    Parent:=DAMonitorForm;
   end;
  with VUMeterBar do
   begin
    Left:=trackBarLeft;
    Top:=trackBarTop-35;
    Width:=trackBarWidth;
    Height:=20;
   end;
  FilenameLabel.Top:=trackBarTop-60;
   
  if (not SetPriorityClass(GetCurrentProcess, REALTIME_PRIORITY_CLASS)) then
    MessageDlg('Could not set real time',mtWarning,[mbOK], 0);
  DropFileTarget1.register(DAMonitorForm);
  DAPlayComponent1.LoadDLL();
  SpectrumVisible(False);
  PlaybackControlVisible(False);
end;

procedure TDAMonitorForm.FormDestroy(Sender: TObject);
begin
  LiveTimer.Enabled := FALSE;
  HRT.Free;
  ACMI.Close;
  ACMO.Close;
  FreeMem(pDataBuf);
  TrackBar1.Free;
  DropFileTarget1.unregister;
end;

procedure TDAMonitorForm.SpectrumVisible(bVal: Boolean);
begin
 ProgressBar1.Visible:=bVal;
 ProgressBar2.Visible:=bVal;
 ProgressBar3.Visible:=bVal;
 ProgressBar4.Visible:=bVal;
 ProgressBar5.Visible:=bVal;
 ProgressBar6.Visible:=bVal;
 ProgressBar7.Visible:=bVal;
 ProgressBar8.Visible:=bVal;
 ProgressBar9.Visible:=bVal;
 ProgressBar10.Visible:=bVal;
 ProgressBar11.Visible:=bVal;
 ProgressBar12.Visible:=bVal;
 Label1.Visible:=bVal;
 Label2.Visible:=bVal;
 Label3.Visible:=bVal;
 Label4.Visible:=bVal;
 Label5.Visible:=bVal;
 Label6.Visible:=bVal;
 Label7.Visible:=bVal;
 Label8.Visible:=bVal;
 Label9.Visible:=bVal;
 Label10.Visible:=bVal;
 Label11.Visible:=bVal;
 Label12.Visible:=bVal;
end;

procedure TDAMonitorForm.PlaybackControlVisible(bVal: Boolean);
begin
 VUMeterBar.Visible:=bVal;
 TrackBar1.Visible:=bVal;
 FilenameLabel.Visible:=bVal;
 BtnStop.Visible:=bVal;
 BtnPlayFile.Enabled:=not bVal;
end;

procedure TDAMonitorForm.SetLiveInput(bVal: Boolean);
var
 dummyFormat: TACMWaveFormat;
begin
 SpeedButtonLiveInput.Down:=bVal;
 if bVal then
  begin
   if not stopped then
     BtnStopClick(Self);
   with dummyFormat.Format do
    begin
     wFormatTag := 1;
     nChannels := channels;
     nSamplesPerSec := samplesPerSec;
     nAvgBytesPerSec:= samplesPerSec;
     nBlockAlign:=1;
     wbitspersample := 16;
     cbSize := SizeOf(TACMWaveFormat);
    end;
   ACMI.Open(dummyFormat);
   if not ACMO.Active then
    begin
     ACMO.NumBuffers := 2;
     ACMO.Open(dummyFormat);
    end;
   LiveTimer.Enabled := TRUE;
  end
 else
  begin
   LiveTimer.Enabled := FALSE;
   ACMI.Close;
   ACMO.Close;
  end;
 SpectrumVisible(bVal);
end;

function TDAMonitorForm.GetNoise(ch: Cardinal) : Integer;
var
 iRetVal: Integer;
begin
 if (ch = 0) then
  begin // Only sync the state to channel 0
   if (eNoiseMode = NoNoise) then
    begin
     noiseStateCnt := samplesPerSec
                      *( random(noiseMaxNormalDuration -noiseMinNormalDuration)
                         +noiseMinNormalDuration);
     noiseFactor := noiseNormalFactor;
     eNoiseMode := NormalNoise;
    end;
   if (eNoiseMode = NormalNoise) then
    begin
     if (noiseStateCnt > 0) then
       Dec(noiseStateCnt)
     else
       eNoiseMode := IncreaseNoise;
    end;
   if (eNoiseMode = IncreaseNoise) then
    begin
     if (noiseStateCnt > 0) then
       Dec(noiseStateCnt)
     else if (noiseFactor > noisePeakFactor) then
      begin
       Dec(noiseFactor);
       noiseStateCnt := noisePeakDuration
                                *samplesPerSec
                                div (noiseNormalFactor -noisePeakFactor);
      end
     else
       eNoiseMode := DecreaseNoise;
    end;
   if (eNoiseMode = DecreaseNoise) then
    begin
     if (noiseStateCnt > 0) then
       Dec(noiseStateCnt)
     else if (noiseFactor < noiseNormalFactor) then
      begin
       Inc(noiseFactor);
       noiseStateCnt := noisePeakDuration
                                *samplesPerSec
                                div (noiseNormalFactor -noisePeakFactor);
      end
     else
       eNoiseMode := NormalNoise;
    end;
  end;
 iRetVal := random(32767);
 GetNoise := iRetVal div noiseFactor;
end;

function TDAMonitorForm.GetFilteredValue(ch: Cardinal; value: Integer) : Integer;
var
 filterConstInx, newValue, locNoise: Integer;
 locVal: Real;
begin
 locVal := value;
 filterConstInx := -1;
 if (filterCode > 0) then
  begin
   if (samplesPerSec = 48000) then
     filterConstInx := 0
   else if (samplesPerSec = 44100) then
     filterConstInx := 1
   else if (samplesPerSec = 32000) then
     filterConstInx := 2
   else if (samplesPerSec = 24000) then
     filterConstInx := 3
   else if (samplesPerSec = 22050) then
     filterConstInx := 4
   else if (samplesPerSec = 16000) then
     filterConstInx := 5;
  end;
 if ((filterCode and filterCodeNoise) <> 0) then
  begin
   if (value > 0) then
     locVal := locVal +GetNoise(ch)
   else
     locVal := locVal -GetNoise(ch);
  end;
 if ((filterCode and filterCodeHP) <> 0) and (filterConstInx >= 0) then
  with filterHP_500_2PoleAtRate[filterConstInx] do
   with fData2[ch] do
    begin
     xv[0] := xv[1]; xv[1] := xv[2];
     xv[2] := locVal / filterGain;
     yv[0] := yv[1]; yv[1] := yv[2];
     yv[2] := (xv[0] + xv[2]) - 2 * xv[1]
           + ( yFactor[0] * yv[0])
           + ( yFactor[1] * yv[1]);
     locVal := yv[2];
    end;
 if ((filterCode and filterCodeLP) <> 0) and (filterConstInx >= 0) then
  with filterLP_3500_8PoleAtRate[filterConstInx] do
   with fData8[ch] do
    begin
     xv[0] := xv[1]; xv[1] := xv[2]; xv[2] := xv[3]; xv[3] := xv[4];
     xv[4] := xv[5]; xv[5] := xv[6]; xv[6] := xv[7]; xv[7] := xv[8];
     xv[8] := locVal / filterGain;
     yv[0] := yv[1]; yv[1] := yv[2]; yv[2] := yv[3]; yv[3] := yv[4];
     yv[4] := yv[5]; yv[5] := yv[6]; yv[6] := yv[7]; yv[7] := yv[8];
     yv[8] :=   (xv[0] + xv[8]) +  8 * (xv[1] + xv[7])
             + 28 * (xv[2] + xv[6]) + 56 * (xv[3] + xv[5])
             + 70 * xv[4]
             + ( yFactor[0] * yv[0]) + ( yFactor[1] * yv[1])
             + ( yFactor[2] * yv[2]) + ( yFactor[3] * yv[3])
             + ( yFactor[4] * yv[4]) + ( yFactor[5] * yv[5])
             + ( yFactor[6] * yv[6]) + ( yFactor[7] * yv[7]);
     locVal := yv[8];
    end;
 GetFilteredValue:=Trunc(locVal);
end;

procedure TDAMonitorForm.FilteredCopy(Source,Dest: pSmallInt; Count: Integer);
var
 i,ch: Integer;
begin
 for i:=0 to (Count div SizeOf(SmallInt))-1 do
  for ch:=0 to channels-1 do
   begin
    Dest^:=GetFilteredValue(ch,Source^);
    Inc(Source);
    Inc(Dest);
  end;
end;

procedure TDAMonitorForm.ACMIBufferFull(Sender: TObject; Data: Pointer;
  Size: Integer);
begin
  FilteredCopy(Data, pDataBuf, Size);
  ACMO.Play(pDataBuf^, Size);

  //  This copies the sampled data to an area for processing.
  //  No DFT's are done here.  The reason is so that even if
  //  the system is loaded, the audio buffer will not overrun.

  //   Also, because windowd serializes messages, there is no problem
  //  with the buffer information being invalid, e.g., working on the
  //  buffer and having new data dumped in it.

  //   Lastly, we are going to use only nDFTSize points out of buffer
  //  size.
  Move(pDataBuf^, Samples, 2*nDFTSize);
end;

{--------------------------sFloor------------------John Mertus Jan 98---}

      Procedure sFloor(src : pDouble; Val : Single; N : Integer);

{  This sets the lowest value of src to VAL.  N is buffer size          }
{									}
{**********************************************************************}
Var
  i : Integer;
BEGIN
  For i := 0 to N-1 Do
    Begin
      If (src^ < Val) Then src^ := Val;
      inc(src);
    End;
END;

{-------------------------preEmph------------------John Mertus Jan 98---}

      Procedure preEmph(src : pDouble; AC1 : Single; N : Integer);

{  This computes the preemphesis of src with value AC1, 0 <=AC1 < 1.        }
{									}
{**********************************************************************}
Var
  i : Integer;
  F, AC0 : Single;
BEGIN
  f := AC1*src^; inc(src);

  For i := 1 to N-1 Do
    Begin
      ac0 := ac1*src^;
      src^ := src^ - F;
      F := ac0;
      inc(src);
    End;
END;

{--------------------CreateDFT--------------------John Mertus Jan 98---}

   Procedure TDAMonitorForm.CreateDFT;

{  This takes the Samples buffer, computes the DFT, computes the log    }
{ magnitude of the spectrum, scales its and leaves the result in the    }
{ Spectrum buffer.                                                      }
{									}
{**********************************************************************}
Var
  FullSize : Integer;
  HalfSize : Integer;
  Order, n     : Integer;

BEGIN
    FullSize := nDFTSize + 2;
    HalfSize := FullSize div 2;

    { Input is 16 bit, convert to floating point}
    nspdbInttoFloat(@Samples, @Spectrum, FullSize, 16, NSP_NOFLAGS);

    { Window the data for smoother looking spectrum }
    preEmph(@Spectrum, 0.98, FullSize);
    nspdWinHamming(@Spectrum, FullSize);

   { Find the next power of 2 = window size }
    n := 1;
    Order := 0;
    While (n < nDFTSize) Do
    Begin
      n := 2*n;
      Order := Order + 1;
    End;

    TimeDFT := HRT.ReadTimer;
    nspdRealFFt(@Spectrum, Order, 1);
    TimeDFT := HRT.ReadTimer - TimeDFT;
    nspzbMag(pDCplx(@Spectrum), @Spectrum, HalfSize);
    sFloor(@Spectrum, 1, HalfSize);
    nspdbLn1(@Spectrum, HalfSize);
    nspdbMpy1(20/ln(10), @Spectrum, HalfSize);
    nspdbAdd1(-40, @Spectrum, HalfSize);    // Noise input level
END;

{------------------DisplaySpectrum----------------John Mertus July 98---}

   Procedure TDAMonitorForm.DisplaySpectrum;

{  This takes the Spectrum buffer, breaks it apart into bins and      }
{ displays the values of the bins.  The freq axis is equal divisions    }
{ of the log.                                                           }
{									}
{**********************************************************************}
const
 noDisplayLevel = 25;
 increaseFactor = 1.4;
 nDL = noDisplayLevel;
 fact = increaseFactor;

Var
  i, j, BinSize : Integer;
  SData : Array[1..NBINS-1] of Double;

Begin
  BinSize := nDFTSize div 10;

  // We do not show the last entry, since it is above audible range
  For i := 1 to NBINS-1 Do
    Begin
      SData[i] := 0;
      For j := ab[i-1] to (ab[i]-1) Do
        SData[i] := SData[i] + Spectrum[j];

      // scale the output to fit 0-100
      SData[i] := 1.5*SData[i]/(ab[i] - ab[i-1]);
    End;

  ProgressBar1.Position := Round(((SData[1])-nDL)*fact);
  ProgressBar2.Position := Round(((SData[2])-nDL)*fact);
  ProgressBar3.Position := Round(((SData[3])-nDL)*fact);
  ProgressBar4.Position := Round(((SData[4])-nDL)*fact);
  ProgressBar5.Position := Round(((SData[5])-nDL)*fact);
  ProgressBar6.Position := Round(((SData[6])-nDL)*fact);
  ProgressBar7.Position := Round(((SData[7])-nDL)*fact);
  ProgressBar8.Position := Round(((SData[8])-nDL)*fact);
  ProgressBar9.Position := Round(((SData[9])-nDL)*fact);
  ProgressBar10.Position := Round(((SData[10])-nDL)*fact);
  ProgressBar11.Position := Round(((SData[11])-nDL)*fact);
  ProgressBar12.Position := Round(((SData[12])-nDL)*fact);
End;

procedure TDAMonitorForm.LiveTimerTimer(Sender: TObject);
Var
   t0, t1 : Double;
begin
   //
   //  Here is where all the work is done, to avoid overflows we turn
   //  the timer off, compute the DFT and spectrum and turn the timer
   //  back on.  This avoids possilbe multiple timer messages.

   If (Not LiveTimer.Enabled) Then Exit;

   LiveTimer.Enabled := FALSE;

   //  Do the DFT's
   HRT.StartTimer;
   CreateDFT;
   t0 := HRT.ReadTimer;

   HRT.StartTimer;
   DisplaySpectrum;
   t1 := HRT.ReadTimer;

   LiveTimer.Enabled := TRUE;
end;

procedure TDAMonitorForm.CheckBoxNoiseClick(Sender: TObject);
begin
 if CheckBoxNoise.Checked then
   filterCode := filterCode or filterCodeNoise
 else
   filterCode := filterCode and not filterCodeNoise
end;

procedure TDAMonitorForm.CheckBoxAMRadioClick(Sender: TObject);
begin
 if CheckBoxAMRadio.Checked then
   filterCode := filterCode or filterCodeLP or filterCodeHP
 else
   filterCode := filterCode and not filterCodeLP and not filterCodeHP;
 RadioImage.Visible:=CheckBoxAMRadio.Checked;
 if RadioImage.Visible then
  begin
   RadioImage.Top:=standardHeight;
   RadioImage.Height:=imageHeight;
   DAMonitorForm.ClientHeight:=standardHeight +imageHeight;
  end
 else
   DAMonitorForm.ClientHeight:=standardHeight;
 CheckBoxNoise.Enabled:=CheckBoxAMRadio.Checked;
 if not CheckBoxNoise.Enabled then
  begin
   saveNoiseEnabled := CheckBoxNoise.Checked;
   CheckBoxNoise.Checked:=False;
   filterCode := filterCode and not filterCodeNoise
  end
 else
   CheckBoxNoise.Checked := saveNoiseEnabled;
 with DAPlayComponent1 do
  begin
   AMRadioLPFilter:=CheckBoxAMRadio.Checked;
   AMRadioHPFilter:=CheckBoxAMRadio.Checked;
   AMRadioNoise:=CheckBoxNoise.Checked;
  end;
(*
 if not stopped then
  begin // Stop and restart at the current position
   if DAPlayComponent1.AbortOk then
    begin
     stopped:=True;
     filePos:=DAPlayComponent1.FinalPosMSec;
     trackBarPos:=(filePos*maxTrackBar) div curFileLen;
     TrackBar1.Position:=trackBarPos;
     listCriticalSection.Enter;
     PrepareFile(0,filePos);
     listCriticalSection.Leave;
    end;
  end;
*)
end;

procedure TDAMonitorForm.SpeedButtonLiveInputClick(Sender: TObject);
begin
 SetLiveInput(SpeedButtonLiveInput.Down);
end;

procedure TDAMonitorForm.PlayFile(const fName: String);
begin
 if FileExists(fName) then
  begin
   if not DAPlayComponent1.SingleThreadPreparePlayback(fName,0,High(Cardinal),0,0)
    or not DAPlayComponent1.SingleThreadPlayOk(0) then
     assert(False)
   else
    begin
     SetLiveInput(False);
     FilenameLabel.Caption:=fName;
     PlaybackControlVisible(True);
     stopped:=False;
    end;
  end;
end;

procedure TDAMonitorForm.DropFileTarget1Drop(Sender: TObject;
  ShiftState: TShiftState; Point: TPoint; var Effect: Integer);
begin
 with DropFileTarget1 do
   PlayFile(files[files.count-1]);
end;

procedure TDAMonitorForm.BtnPlayFileClick(Sender: TObject);
begin
 OpenDialog1.Title := 'Play Audio File';
 if OpenDialog1.Execute then
   PlayFile(OpenDialog1.FileName);
end;

procedure TDAMonitorForm.TrackBar1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 desiredTrackbarPos:=TrackBar1.Position;
 JumpToPos;
 bDragPos:=False;
end;

procedure TDAMonitorForm.TrackBar1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
 bDragPos:=True;
end;

procedure TDAMonitorForm.JumpToPos();
var
 newPos: Cardinal;
begin
// EnableControls(False);
 bRepositioning:=True;
 Screen.Cursor:=crHourglass;
 newPos:=(Cardinal(desiredTrackbarPos)*Cardinal(curFileLen)) div maxTrackBar;
 DAPlayComponent1.Jump(newPos-filePos);
 trackBarPos:=TrackBar1.Position;
 desiredTrackbarPos:=-1;
end;

procedure TDAMonitorForm.DAPlayComponent1RemainingTime(Sender: TObject;
  val: Cardinal);
begin
 if (filePos <> 0) and (val+filePos <> 0) and not bDragPos then
  begin
   curFileLen:=val+filePos;
   trackBarPos:=(filePos *maxTrackBar) div curFileLen;
   if (trackBarPos <> TrackBar1.Position) then
     TrackBar1.Position:=trackBarPos;
  end;
end;

procedure TDAMonitorForm.DAPlayComponent1FileposTime(Sender: TObject;
  val: Cardinal);
begin
 if not stopped then
   filePos:=val;
end;

procedure TDAMonitorForm.DAPlayComponent1EndPlayback(Sender: TObject;
  val: Cardinal);
begin
 if not stopped then
   filePos := 0;
  // Reset the jump state in case we jumped too far
 DAPlayComponent1JumpOk(Sender);
 PlaybackControlVisible(False);
end;

procedure TDAMonitorForm.DAPlayComponent1TerminateItem(Sender: TObject;
  val: Cardinal);
begin
 PlaybackControlVisible(False);
end;

procedure TDAMonitorForm.DAPlayComponent1JumpOk(Sender: TObject);
begin
 if bRepositioning then
  begin
   bRepositioning:=False;
   Screen.Cursor:=crDefault;
  end;
end;

procedure TDAMonitorForm.DAPlayComponent1VULevelChange(Sender: TObject;
  val: Integer);
begin
 VUMeterBar.Position := val;
end;

procedure TDAMonitorForm.BtnStopClick(Sender: TObject);
begin
 if DAPlayComponent1.AbortOk then
  begin
   stopped:=True;
   filePos:=0;
   trackBarPos:=0;
   TrackBar1.Position:=0;
  end;
end;

end.

